"""test_connectionLists.py: 

Test connectionList in SparseMsg.

"""

from __future__ import print_function
    
__author__           = "Dilawar Singh"
__copyright__        = "Copyright 2016, Dilawar Singh"
__credits__          = ["NCBS Bangalore"]
__license__          = "GNU GPL"
__version__          = "1.0.0"
__maintainer__       = "Dilawar Singh"
__email__            = "dilawars@ncbs.res.in"
__status__           = "Development"

import moose

params = {
        'Rm': 1e10,
        'Cm': 1e-11,
        'randInputRate': 10,
        'randRefractTime': 0.005,
        'numInputs': 100,
        'numInhib': 10,
        'inhibRefractTime': 0.005,
        'inhibThresh': -0.04,
        'numOutput': 10,
        'outputRefractTime': 0.005,
        'outputThresh': 5.00e6,
        'stimToInhProb': 0.01,
        'stimToInhSeed': 11234,
        'inhVreset': -0.06,
        'stimToOutProb': 0.05,
        'stimToOutSeed': 1234,
        'inhToOutProb': 0.5,
        'inhToOutSeed': 1234,
        'runtime': 2.0,
        'wtStimToOut': 0.001,
        'wtStimToInh': 0.002,
        'wtInhToOut': -0.002,
}


def makeGlobalBalanceNetwork():
    stim = moose.RandSpike( '/model/stim', params['numInputs'] )
    inhib = moose.LIF( '/model/inhib', params['numInhib'] )
    insyn = moose.SimpleSynHandler(inhib.path + '/syns', params['numInhib'])
    moose.connect( insyn, 'activationOut', inhib, 'activation', 'OneToOne' )
    output = moose.LIF( '/model/output', params['numOutput'] )
    outsyn = moose.SimpleSynHandler(output.path+'/syns',params['numOutput'])
    moose.connect(outsyn, 'activationOut', output, 'activation', 'OneToOne')
    outInhSyn = moose.SimpleSynHandler(output.path+'/inhsyns',params['numOutput'])
    moose.connect(outInhSyn, 'activationOut', output, 'activation', 'OneToOne')

    iv = moose.vec( insyn.path + '/synapse' )
    ov = moose.vec( outsyn.path + '/synapse' )
    oiv = moose.vec( outInhSyn.path + '/synapse' )

    assert len(iv) == 0
    assert len(ov) == 0
    assert len(oiv) == 0

    temp = moose.connect( stim, 'spikeOut', iv, 'addSpike', 'Sparse' )
    inhibMatrix = moose.element( temp )
    inhibMatrix.setRandomConnectivity( 
            params['stimToInhProb'], params['stimToInhSeed'] )
    cl = inhibMatrix.connectionList

    # This can change when random-number generator changes.
    # This was before we used c++11 <random> to generate random numbers. This
    # test has changes on Tuesday 31 July 2018 11:12:35 AM IST
    #  expectedCl = [ 1,4,13,13,26,42,52,56,80,82,95,97,4,9,0,9,4,8,0,6,1,6,6,7]
    expectedCl=[0,6,47,50,56,67,98,2,0,3,5,4,8,3]

    assert list(cl) == expectedCl, "Expected %s, got %s" % (expectedCl, cl)

    temp = moose.connect( stim, 'spikeOut', ov, 'addSpike', 'Sparse' )
    excMatrix = moose.element( temp )
    excMatrix.setRandomConnectivity( 
            params['stimToOutProb'], params['stimToOutSeed'] )

    temp = moose.connect( inhib, 'spikeOut', oiv, 'addSpike', 'Sparse' )
    negFFMatrix = moose.element( temp )
    negFFMatrix.setRandomConnectivity( 
            params['inhToOutProb'], params['inhToOutSeed'] )

    # print("ConnMtxEntries: ", inhibMatrix.numEntries, excMatrix.numEntries, negFFMatrix.numEntries)
    got = (inhibMatrix.numEntries, excMatrix.numEntries, negFFMatrix.numEntries)
    expected = (7, 62, 55)
    assert expected == got, "Expected %s, Got %s" % (expected,got)

    cl = negFFMatrix.connectionList
    numInhSyns = [ ]
    niv = 0
    nov = 0
    noiv = 0
    for i in moose.vec( insyn ):
        niv += i.synapse.num
        numInhSyns.append( i.synapse.num )
        if i.synapse.num > 0:
            i.synapse.weight = params['wtStimToInh']

    #  expected = [2,1,0,0,2,0,3,1,1,2]
    expected = [1, 0, 1, 2, 1, 1, 0, 0, 1, 0]
    assert numInhSyns == expected, "Expected %s, got %s" % (expected,numInhSyns)

    for i in moose.vec( outsyn ):
        print('111', i)
        nov += i.synapse.num
        if i.synapse.num > 0:
            i.synapse.weight = params['wtStimToOut']
    for i in moose.vec( outInhSyn ):
        noiv += i.synapse.num
        #print i.synapse.num
        if i.synapse.num > 0:
            i.synapse.weight = params['wtInhToOut']

    print("SUMS: ", sum( iv.numField ), sum( ov.numField ), sum( oiv.numField ))
    assert [1, 64, 25] == [sum( iv.numField ), sum( ov.numField ), sum( oiv.numField )]
    print("SUMS2: ", niv, nov, noiv)
    assert [7, 62, 55] ==  [ niv, nov, noiv ]
    print("SUMS3: ", sum( insyn.vec.numSynapses ), sum( outsyn.vec.numSynapses ), sum( outInhSyn.vec.numSynapses ))
    assert [7,62,55] == [ sum( insyn.vec.numSynapses ), sum( outsyn.vec.numSynapses ), sum( outInhSyn.vec.numSynapses ) ]

    # print(oiv.numField)
    # print(insyn.vec[1].synapse.num)
    # print(insyn.vec.numSynapses)
    # print(sum( insyn.vec.numSynapses ))
    # niv = iv.numSynapses
    # ov = iv.numSynapses

    sv = moose.vec( stim )
    sv.rate = params['randInputRate']
    sv.refractT = params['randRefractTime']
    #moose.showfield( sv[7] )

    inhib.vec.thresh = params['inhibThresh']
    inhib.vec.Rm = params['Rm']
    inhib.vec.Cm = params['Cm']
    inhib.vec.vReset = params['inhVreset']
    inhib.vec.refractoryPeriod = params['inhibRefractTime']
    output.vec.thresh = params['outputThresh']
    output.vec.Rm = params['Rm']
    output.vec.Cm = params['Cm']
    output.vec.refractoryPeriod = params['outputRefractTime']
    otab = moose.Table( '/model/otab', params['numOutput'] )
    moose.connect( otab, 'requestOut', output, 'getVm', 'OneToOne' )
    itab = moose.Table( '/model/itab', params['numInhib'] )
    moose.connect( itab, 'requestOut', inhib, 'getVm', 'OneToOne' )
    return inhib, output

def makeRaster( network ):
    raster = moose.Table(network.path + '/raster', network.numData)
    moose.connect(network, 'spikeOut', raster, 'spike', 'OneToOne')
    return raster

def test_connection_list():
    moose.Neutral( '/model' )
    inhib, output = makeGlobalBalanceNetwork()
    iraster = makeRaster( inhib )
    makeRaster( output )
    moose.reinit()
    moose.start( params['runtime'] )
    for i in range( len(iraster.vec) ):
        print( iraster.vec[i].vector )

if __name__ == '__main__':
    test_connection_list()



# -*- coding: utf-8 -*-
# __init__.py ---
#
# Filename: __init__.py
# Description:
# Author: subha
# Maintainer:
# Created: Sun Apr 17 13:55:30 2016 (-0400)
# Version:
# Last-Updated: Sun Apr 17 14:59:06 2016 (-0400)
#           By: subha
#     Update #: 20
# URL:
# Keywords:
# Compatibility:
#
#

# Commentary:
#
#
#
#

# Change log:
#
#
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street, Fifth
# Floor, Boston, MA 02110-1301, USA.
#
#

# Code:
"""neuroml2 submodule handles NeuroML2 models in MOOSE.

NML2Reader class allows reading of NeuroML2 cell models with
Hodgkin-Huxley type ion channels.

The [Ca2+] dependent channel reading may be incomplete.

Some functions for converting in-memory MOOSE models into NeuroML2 are
available in converter.py. But the complete cell-model writing is not
fully implemented. Hence this is not included in __all__.

"""
from __future__ import absolute_import, division, print_function
import moose.utils as mu

__author__ = 'Subhasis Ray'

from .reader import NML2Reader

# __init__.py ends here



# -*- coding: utf-8 -*-
'''
*******************************************************************
 * File:            chemConnectUtil.py
 * Description:
 * Author:          HarshaRani
 * E-mail:          hrani@ncbs.res.in
 ********************************************************************/
/**********************************************************************
** This program is part of 'MOOSE', the
** Messaging Object Oriented Simulation Environment,
** also known as GENESIS 3 base code.
**           copyright (C) 2003-2017 Upinder S. Bhalla. and NCBS
Created : Friday May 27 12:19:00 2016(+0530)
Version
Last-Updated: Mon Aug 7 17:02:00 2017(+0530)
          By: HarshaRani
**********************************************************************/
/****************************

Aug 7: cleaned up space
'''

import moose

pygraphvizFound_ = True
try:
    import pygraphviz as pgv
except Exception as e:
    pygraphvizFound_ = False

def autoCoordinates(meshEntry,srcdesConnection):
    global pygraphvizFound_
    positionInfo = {}

    if not pygraphvizFound_:
        print( '[warn] python-pygraphviz could not be found.' )
        print( '\tMOOSE Install pygraphviz to use this feature' )
        return positionInfo

    if meshEntry:
        #G = nx.Graph()
        G = pgv.AGraph()

        for cmpt,memb in list(meshEntry.items()):
            for enzObj in find_index(memb,'enzyme'):
                #G.add_node(enzObj.path)
                G.add_node(enzObj.path,label='',shape='ellipse',color='',style='filled',fontname='Helvetica',fontsize=12,fontcolor='blue')
        for cmpt,memb in list(meshEntry.items()):
            for poolObj in find_index(memb,'pool'):
                #G.add_node(poolObj.path)
                G.add_node(poolObj.path,label = poolObj.name,shape = 'box',color = '',style = 'filled',fontname = 'Helvetica',fontsize = 12,fontcolor = 'blue')
            for cplxObj in find_index(memb,'cplx'):
                G.add_node(cplxObj.path)
                G.add_node(cplxObj.path,label = cplxObj.name,shape = 'box',color = '',style = 'filled',fontname = 'Helvetica',fontsize = 12,fontcolor = 'blue')
                #G.add_edge((cplxObj.parent).path,cplxObj.path)
            for reaObj in find_index(memb,'reaction'):
                #G.add_node(reaObj.path)
                G.add_node(reaObj.path,label='',shape='circle',color='')

        for inn,out in list(srcdesConnection.items()):
            if (inn.className =='ZombieReac'): arrowcolor = 'green'
            elif(inn.className =='ZombieEnz'): arrowcolor = 'red'
            else: arrowcolor = 'blue'
            if isinstance(out,tuple):
                if len(out[0])== 0:
                    print( inn.className + ':' +inn.name + "  doesn't have input message")
                else:
                    for items in (items for items in out[0] ):
                        G.add_edge(moose.element(items[0]).path,inn.path)
                if len(out[1]) == 0:
                    print(inn.className + ':' + inn.name + " doesn't have output mssg")
                else:
                    for items in (items for items in out[1] ):
                        G.add_edge(inn.path,moose.element(items[0]).path)
            elif isinstance(out,list):
                if len(out) == 0:
                    print ("Func pool doesn't have sumtotal")
                else:
                    for items in (items for items in out ):
                        G.add_edge(moose.element(items[0]).path,inn.path)

        # if int( nx.__version__.split( '.' )[-1] ) >= 11:
        #     position = nx.spring_layout( G )
        # else:
        #     position = nx.graphviz_layout(G, prog = 'dot')
        # for item in position.items():
        #     xy = item[1]
        #     ann = moose.Annotator(item[0]+'/info')
        #     ann.x = xy[0]
        #     ann.y = xy[1]
        #     positioninfo[(moose.element(item[0]))] ={'x':float(xy[0]),'y':float(xy[1])}

        G.layout()
        for n in G.nodes():
            value = str(n.attr['pos'])
            valuelist = (value.split(','))
            positionInfo[(moose.element(n))] ={'x':float(valuelist[0]),'y':float(valuelist[1])}
            ann = moose.Annotator(moose.element(n).path+'/info')
            ann.x = float(valuelist[0])
            ann.y = float(valuelist[1])

    return positionInfo

def find_index(value, key):
    """ Value.get(key) to avoid expection which would raise if empty value in dictionary for a given key """
    if value.get(key) != None:
        return value.get(key)
    else:
        raise ValueError('no dict with the key found')



# -*- coding: utf-8 -*-
# Script to test all import modules.

import moose

try:
    import libsbml
except ImportError as e:
    print(e)
    pass

def library1():
    import moose.genesis 
    import moose.SBML 
    import moose.chemMerge
    import moose.utils 
    import moose.network_utils
    print('done')

    p1 = moose.le()
    a = moose.Pool('/a')
    for i in range(10):
        moose.Pool('/a/p%d'%i)
    p2 = moose.le()
    assert set(p2) - set(p1) == set(['/a']), set(p2) - set(p1)
    aa = moose.le(a)
    assert len(aa) == 10

    try:
        moose.showfield('/x')
    except ValueError:
        pass

    moose.showfield('/a')

def test_library():
    library1()

def main():
    test_library()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""test_table_streaming_support.py:

Test the streaming support in moose.Table.

"""

__author__           = "Dilawar Singh"
__copyright__        = "Copyright 2016, Dilawar Singh"
__credits__          = ["NCBS Bangalore"]
__license__          = "GNU GPL"
__version__          = "1.0.0"
__maintainer__       = "Dilawar Singh"
__email__            = "dilawars@ncbs.res.in"
__status__           = "Development"

import os
import sys
import moose
import numpy as np
print( '[INFO] Using moose form %s' % moose.__file__ )

def print_table( table ):
    msg = ""
    msg += " datafile : %s" % table.datafile
    msg += " useStreamer: %s" % table.useStreamer
    msg += ' Path: %s' % table.path
    print( msg )

def test_small( ):
    moose.CubeMesh( '/compt' )
    r = moose.Reac( '/compt/r' )
    a = moose.Pool( '/compt/a' )
    a.concInit = 1
    b = moose.Pool( '/compt/b' )
    b.concInit = 2
    c = moose.Pool( '/compt/c' )
    c.concInit = 0.5
    moose.connect( r, 'sub', a, 'reac' )
    moose.connect( r, 'prd', b, 'reac' )
    moose.connect( r, 'prd', c, 'reac' )
    r.Kf = 0.1
    r.Kb = 0.01

    tabA = moose.Table2( '/compt/a/tabA' )
    #  tabA.format = 'npy'
    tabA.useStreamer = True   # Setting format alone is not good enough

    # Setting datafile enables streamer.
    tabB = moose.Table2( '/compt/b/tabB' )
    tabB.datafile = 'table2.npy'

    tabC = moose.Table2( '/compt/c/tabC' )
    tabC.datafile = 'tablec.csv'

    moose.connect( tabA, 'requestOut', a, 'getConc' )
    moose.connect( tabB, 'requestOut', b, 'getConc' )
    moose.connect( tabC, 'requestOut', c, 'getConc' )

    moose.reinit( )
    [ print_table( x) for x in [tabA, tabB, tabC] ]
    runtime = 1000
    print( 'Starting moose for %d secs' % runtime )
    moose.start( runtime, 1 )

    # Now read the numpy and csv and check the results.
    a = np.loadtxt( tabA.datafile, skiprows=1 )
    b = np.load( 'table2.npy' )
    c = np.loadtxt( 'tablec.csv', skiprows=1 )
    assert (len(a) == len(b) == len(c))
    print( ' MOOSE is done' )

def buildLargeSystem(useStreamer = False):
    # create a huge system.
    if moose.exists('/comptB'):
        moose.delete('/comptB')
    moose.CubeMesh( '/comptB' )

    tables = []
    for i in range(300):
        r = moose.Reac('/comptB/r%d'%i)
        a = moose.Pool('/comptB/a%d'%i)
        a.concInit = 10.0
        b = moose.Pool('/comptB/b%d'%i) 
        b.concInit = 2.0
        c = moose.Pool('/comptB/c%d'%i)
        c.concInit = 0.5
        moose.connect( r, 'sub', a, 'reac' )
        moose.connect( r, 'prd', b, 'reac' )
        moose.connect( r, 'prd', c, 'reac' )
        r.Kf = 0.1
        r.Kb = 0.01

        # Make table name large enough such that the header is larger than 2^16
        # . Numpy version 1 can't handle such a large header. If format 1 is
        # then this test will fail.
        t = moose.Table2('/comptB/TableO1%d'%i + 'abc'*100)
        moose.connect(t, 'requestOut', a, 'getConc')
        tables.append(t)

    if useStreamer:
        s = moose.Streamer('/comptB/streamer')
        s.datafile = 'data2.npy'
        print("[INFO ] Total tables %d" % len(tables))

        # Add tables using wilcardFind.
        s.addTables(moose.wildcardFind('/comptB/##[TYPE=Table2]'))

        print("Streamer has %d table" % s.numTables)
        assert s.numTables == len(tables), (s.numTables, len(tables))

    moose.reinit()
    moose.start(10)

    if useStreamer:
        # load the data
        data = np.load(s.datafile)
        header = str(data.dtype.names)
        assert len(header) > 2**16
    else:
        data = { x.columnName : x.vector for x in tables }
    return data

def test_large_system():
    # Get data without streamer and with streamer.
    # These two must be the same.
    X = buildLargeSystem(False)   # without streamer
    Y = buildLargeSystem(True)    # with streamer.

    # X has no time.
    assert len(X) == len(Y.dtype.names)-1, (len(X), Y.dtype)

    # same column names.
    xNames = list(X.keys())
    yNames = list(Y.dtype.names)
    assert set(yNames) - set(xNames)  == set(['time']), (yNames, xNames)

    # Test for equality in some tables.
    for i in range(1, 10):
        a, b = Y[xNames[i]], X[xNames[i]]
        assert a.shape == b.shape, (a.shape, b.shape)
        assert (a == b).all(), (a-b)


def main( ):
    test_small( )
    test_large_system()
    print( '[INFO] All tests passed' )

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#########################################################################
## This program is part of 'MOOSE', the
## Messaging Object Oriented Simulation Environment.
##           Copyright (C) 2018 Upinder S. Bhalla. and NCBS
## It is made available under the terms of the
## GNU General Public License version 3 or later.
## See the file COPYING.LIB for the full notice.
#########################################################################


import math
import numpy as np
import moose
import moose.fixXreacs as fixXreacs

def makeModel():
    # create container for model
    num = 1 # number of compartments
    model = moose.Neutral( '/model' )
    compartment = moose.CylMesh( '/model/compartment' )
    compartment.x1 = 1.0e-6 # Set it to a 1 micron single-voxel cylinder

    # create molecules and reactions
    s = moose.Pool( '/model/compartment/s' )
    t = moose.Pool( '/model/compartment/t' )
    rXfer = moose.Reac( '/model/compartment/rXfer' )
    #####################################################################
    # Put in endo compartment. Add molecule s
    endo = moose.EndoMesh( '/model/endo' )
    endo.isMembraneBound = True
    endo.surround = compartment
    es = moose.Pool( '/model/endo/s' )
    et = moose.Pool( '/model/endo/t' )
    #####################################################################
    moose.connect( rXfer, 'sub', s, 'reac' )
    moose.connect( rXfer, 'sub', t, 'reac' )
    moose.connect( rXfer, 'prd', es, 'reac' )
    moose.connect( rXfer, 'prd', et, 'reac' )
    rXfer.Kf = 0.02 # 0.02/mM/sec
    rXfer.Kb = 0.02 # 0.02/mM/sec

    #####################################################################
    fixXreacs.fixXreacs( '/model' )
    #fixXreacs.restoreXreacs( '/model' )
    #fixXreacs.fixXreacs( '/model' )
    #####################################################################
    # Make solvers
    ksolve = moose.Ksolve( '/model/compartment/ksolve' )
    dsolve = moose.Dsolve( '/model/dsolve' )
    eksolve = moose.Ksolve( '/model/endo/ksolve' )
    edsolve = moose.Dsolve( '/model/endo/dsolve' )

    stoich = moose.Stoich( '/model/compartment/stoich' )
    stoich.compartment = compartment
    stoich.ksolve = ksolve
    stoich.dsolve = dsolve
    stoich.path = "/model/compartment/##"
    assert( dsolve.numPools == 4 )
    s.vec.concInit = [1.0]*num
    t.vec.concInit = [1.0]*num

    estoich = moose.Stoich( '/model/endo/stoich' )
    estoich.compartment = endo
    estoich.ksolve = eksolve
    estoich.dsolve = edsolve
    estoich.path = "/model/endo/##"
    assert( edsolve.numPools == 2 )

    edsolve.buildMeshJunctions( dsolve )

    plot1 = moose.Table2( '/model/plot1' )
    plot2 = moose.Table2( '/model/plot2' )
    moose.connect( '/model/plot1', 'requestOut', s, 'getN' )
    moose.connect( '/model/plot2', 'requestOut', es, 'getN' )
    plot3 = moose.Table2( '/model/plot3' )
    plot4 = moose.Table2( '/model/plot4' )
    moose.connect( '/model/plot3', 'requestOut', s, 'getConc' )
    moose.connect( '/model/plot4', 'requestOut', es, 'getConc' )

def almostEq( a, b ):
    #print a, b, (a-b)/(a+b)
    return abs(a-b)/(a+b) < 5e-5

def test_xreac6():
    for i in range( 10, 18):
        moose.setClock( i, 0.01 )
    runtime = 100
    makeModel()
    moose.reinit()
    moose.start( runtime )
    assert( almostEq( moose.element( 'model/compartment/s' ).conc,
        moose.element( '/model/endo/s' ).conc ) )
    moose.delete( '/model' )

def main():
    test_xreac6()

# Run the 'main' if this script is executed standalone.
if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Author: Subha
# Maintainer: Dilawar Singh
# Created: Tue May 21 16:34:45 2013 (+0530)
# This test is fragile.

from __future__ import print_function, division, absolute_import
import numpy as np
import unittest
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import moose.neuroml2.hhfit as hhfit

np.random.seed(10)


class TestFindRateFn(unittest.TestCase):
    def setUp(self):
        self.vmin = -120e-3
        self.vmax = 40e-3
        self.vdivs = 640
        self.v_array = np.linspace(self.vmin, self.vmax, self.vdivs + 1)
        # Parameters for sigmoid function - from traub2005, NaF->m_inf
        p_sigmoid = (1.0, 1 / -10e-3, -38e-3, 0.0)
        self.sigmoid = p_sigmoid[0] / (
            1.0 + np.exp(p_sigmoid[1] *
                         (self.v_array - p_sigmoid[2]))) + p_sigmoid[3]
        self.p_sigmoid = p_sigmoid
        # Parameters for exponential function - from traub2005, KC->n_inf
        p_exp = (2e3, 1 / -27e-3, -53.5e-3, 0.0)
        self.exp = p_exp[0] * np.exp(p_exp[1] *
                                     (self.v_array - p_exp[2])) + p_exp[3]
        self.p_exp = p_exp
        # Parameters for linoid function: alpha_n from original Hodgkin-Huxley K channel.
        p_linoid = (-0.01 * 1e3, -1 / 10e-3, 10e-3, 0.0)
        self.linoid = p_linoid[3] + p_linoid[0] * \
                (self.v_array - p_linoid[2]) / (np.exp(p_linoid[1] * (self.v_array - p_linoid[2])) - 1)
        self.p_linoid = p_linoid
        # This is tau_m of transient Ca2+ current (eq. 7) from
        # Huguenard and McCormick, J Neurophysiol, 68:1373-1383,
        # 1992.;
        #1e-3 * (0.612 + 1 / (np.exp((self.v_array*1e3 + 132)/-16.7) + np.exp((self.v_array*1e3 + 16.8)/18.2)))
        p_dblexp = (1e-3, -1 / 16.7e-3, -132e-3, 1 / 18.2e-3, -16.8e-3,
                    0.612e-3)
        self.dblexp = p_dblexp[5] + p_dblexp[0] / (
            np.exp(p_dblexp[1] * (self.v_array - p_dblexp[2])) +
            np.exp(p_dblexp[3] * (self.v_array - p_dblexp[4])))
        self.p_dblexp = p_dblexp

    def test_sigmoid(self):
        print('Testing sigmoid')
        fn, params = hhfit.find_ratefn(self.v_array, self.sigmoid)
        print('Sigmoid params original:', self.p_sigmoid, 'detected:', params)
        self.assertEqual(hhfit.sigmoid, fn)
        rms_error = np.sqrt(
            np.mean((self.sigmoid - fn(self.v_array, *params))**2))
        self.assertAlmostEqual(rms_error / max(abs(self.sigmoid)),
                               0.0,
                               places=3)
        plt.plot(self.v_array, self.sigmoid, 'y-', self.v_array,
                 hhfit.sigmoid(self.v_array, *self.p_sigmoid), 'b--',
                 self.v_array, fn(self.v_array, *params), 'r-.')
        plt.legend('original sigmoid %s fitted %s' % (self.p_sigmoid, fn))
        plt.savefig("__test_sigmoid.png")

    def test_exponential(self):
        print('Testing exponential')
        fn, params = hhfit.find_ratefn(self.v_array, self.exp)
        print('Exponential params original:', self.p_exp, 'detected:', params)
        if params is not None:
            # The `find_ratefn` might return a parameter array for different
            # function sometimes. exponential takes only upto 5 parameters. 
            fnval = hhfit.exponential(self.v_array, *params[:4])
            self.assertEqual(hhfit.exponential, fn)
            # The same exponential can be satisfied by an infinite number
            # of parameter values. Hence we cannot compare the parameters,
            # but only the fit
            rms_error = np.sqrt(np.sum((self.exp - fnval)**2))
            print(rms_error, rms_error / max(self.exp))
            self.assertAlmostEqual(rms_error / max(self.exp), 0.0, places=3)
            plt.plot(self.v_array, self.exp, 'y-', self.v_array,
                     hhfit.exponential(self.v_array, *self.p_exp), 'b--',
                     self.v_array, fnval, 'r-.')
            plt.legend('original exp %s fitted %s' % (self.p_exp, fn))
            out = "__test_exponential.png"
            plt.savefig(out)
            print('Plot is saved saved to %s' % out)
        else:
            print("[INFO ] Failed find a suitable approximation...")
            

    def test_linoid(self):
        print('Testing linoid')
        fn, params = hhfit.find_ratefn(self.v_array, self.linoid)
        if params is not None:
            print('Linoid params original:', self.p_linoid, 'detected:', params)
            self.assertEqual(hhfit.linoid, fn)
            fnval = fn(self.v_array, *params)
            rms_error = np.sqrt(np.mean((self.linoid - fnval)**2))
            self.assertAlmostEqual(rms_error / max(self.linoid), 0.0, places=3)
            plt.plot(self.v_array, self.linoid, 'y-', self.v_array,
                     hhfit.linoid(self.v_array, *self.p_linoid), 'b--',
                     self.v_array, fn(self.v_array, *params), 'r-.')
            plt.legend('Original linoid %s fitted %s' % (self.p_linoid, fn))
            out = "__test_linoid.png"
            plt.savefig(out)
            print('Plot is saved saved to %s' % out)
        else:
            print('Failed to find a  suitable fit.')

    def test_dblexponential(self):
        print('Testing double exponential')
        fn, params = hhfit.find_ratefn(self.v_array, self.dblexp)
        fnval = fn(self.v_array, *params)
        plt.plot(self.v_array, self.dblexp, 'y-', self.v_array,
                 hhfit.double_exp(self.v_array, *self.p_dblexp), 'b--',
                 self.v_array, fnval, 'r-.')
        self.assertEqual(hhfit.double_exp, fn)
        rms_error = np.sqrt(np.mean((self.dblexp - fnval)**2))
        print(params, rms_error)
        self.assertAlmostEqual(rms_error / max(self.dblexp), 0.0, places=3)
        plt.legend('Original dblexp %s, fitted %s' %(self.dblexp, fn))
        out = "__test_dblexponential.png"
        plt.savefig(out)
        print('Plot is saved saved to %s' % out)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

# Some network analysis utilities.

__author__           = "Dilawar Singh"
__copyright__        = "Copyright 2018-19, NCBS Bangalore"
__maintainer__       = "Dilawar Singh"
__email__            = "dilawars@ncbs.res.in"

import sys
import hashlib
import moose._moose as _moose
import re

import logging
logger_ = logging.getLogger('moose.utils.graph')

pathPat_ = re.compile(r'.+?\[\d+\]$')

def morphologyToGraphviz(filename=None, pat='/##[TYPE=Compartment]'):
    '''Write Electrical network to a dot graph.

    Params:

    :filename: Default None. Write graphviz file to this path. If None, write to
        stdout.
    :pat: Compartment path. By default, search for all moose.Compartment.
    '''

    def fix(path):
        '''Fix a given path so it can be written to a graphviz file'''
        # If no [0] is at end of the path then append it.
        global pathPat_
        if not pathPat_.match(path):
            path = path + '[0]'
        return path

    compList = _moose.wildcardFind(pat)
    if not compList:
        logger_.warn("No compartment found")

    dot = []
    dot.append("digraph G {")
    dot.append("\tconcentrate=true;")
    for c in compList:
        if c.neighbors['raxial']:
            for n in c.neighbors['raxial']:
                lhs = fix(c.path)
                rhs = fix(n.path)
                dot.append('\t"{}" -> "{}";'.format(lhs, rhs))
        elif c.neighbors['axial']:
            for n in c.neighbors['axial']:
                lhs = fix(c.path)
                rhs = fix(n.path)
                dot.append('\t"{}" -> "{}" [dir=back];'.format(lhs, rhs))
        else:
            p = fix(c.path)
            dot.append('\t"{}"'.format(p))

    dot.append('}')
    dot = '\n'.join(dot)
    if not filename:
        print(dot)
    else:
        with open(filename, 'w') as graphviz:
            logger_.info("Writing compartment topology to file {}".format(filename))
            graphviz.write(dot)
    return True


def chemicalReactionNetworkToGraphviz(compt, path=None):
    """Write chemical reaction network to a graphviz file.

    :param compt: Given compartment.
    :param filepath: Save to this filepath. If None, write to stdout.
    """
    dot = _crn(compt)
    if path is None:
        print(dot, file=sys.stdout)
        return
    with open(path, 'w') as f:
        f.write(dot)

# aliases
crnToDot = chemicalReactionNetworkToGraphviz
crnToGraphviz = chemicalReactionNetworkToGraphviz

def _fixLabel(name):
    name = name.replace('*', 'star')
    name = name.replace('.', '_')
    return "\"{}\"".format(name)

def _addNode(n, nodes, dot):
    node = hashlib.sha224(n.path.encode()).hexdigest()
    nodeType = 'pool'
    if isinstance(n, _moose.Reac) or isinstance(n, _moose.ZombieReac):
        node = 'r'+node
        nodeType = 'reac'
    else:
        node = 'p'+node
    if node in nodes:
        return node

    nLabel = n.name
    if nodeType == 'reac':
        nLabel = "kf=%g kb=%g"%(n.Kf, n.Kb)
        dot.append('\t%s [label="%s", kf=%g, kb=%g, shape=rect]' % (
            node, nLabel, n.Kf, n.Kb))
    else:
        dot.append('\t%s [label="%s", concInit=%g]' % (
            node, nLabel, n.concInit))
    return node

def _crn(compt):
    nodes = {}
    reacs = _moose.wildcardFind(compt.path+'/##[TYPE=Reac]')
    reacs += _moose.wildcardFind(compt.path+'/##[TYPE=ZombieReac]')
    dot = ['digraph %s {\n\t overlap=false' % compt.name ]
    for r in reacs:
        rNode = _addNode(r, nodes, dot)
        for s in r.neighbors['sub']:
            sNode = _addNode(s, nodes, dot)
            dot.append('\t%s -> %s' % (sNode, rNode))
        for p in r.neighbors['prd']:
            pNode = _addNode(p, nodes, dot)
            dot.append('\t%s -> %s' % (rNode, pNode))
    dot.append('}')
    return '\n'.join(dot)



# Rdesigneur example 4.1
# Making an axon with a propagating action potential.
# Turned into a doctest

import matplotlib
matplotlib.use('Agg')
import numpy as np
import moose
import rdesigneur as rd

numAxonSegments = 200
comptLen = 10e-6
comptDia = 1e-6
RM = 1.0
RA = 10.0
CM = 0.01

expected = np.array([-0.065     , -0.06983536, -0.07267512, -0.07384024, -0.07422599,
           -0.07426641, -0.07057581, -0.06788169, -0.06541139, -0.06203896,
           -0.04844496,  0.02185485, -0.02780464, -0.07129777, -0.07606423,
           -0.07568317, -0.07514661, -0.07449245, -0.07373675, -0.07290267,
           -0.07201665, -0.07110338, -0.07018217, -0.06926533, -0.06835791,
           -0.06745804, -0.0665565 , -0.06563355, -0.06464782, -0.06349831,
           -0.06186082, -0.05798547,  0.00504934, -0.00513958, -0.04914883,
           -0.07588121, -0.07586509, -0.07538907, -0.07478491, -0.07407192,
           -0.0732706 ])

def makeAxonProto():
    axon = moose.Neuron( '/library/axon' )
    prev = rd.buildCompt( axon, 'soma', RM = RM, RA = RA, CM = CM, dia = 10e-6, x=0, dx=comptLen)
    theta = 0
    x = comptLen
    y = 0.0

    for i in range( numAxonSegments ):
        dx = comptLen * np.cos( theta )
        dy = comptLen * np.sin( theta )
        r = np.sqrt( x * x + y * y )
        theta += comptLen / r
        compt = rd.buildCompt( axon, 'axon' + str(i), RM = RM, RA = RA, CM = CM, x = x, y = y, dx = dx, dy = dy, dia = comptDia )
        moose.connect( prev, 'axial', compt, 'raxial' )
        prev = compt
        x += dx
        y += dy
    
    return axon

def test_axon_propagation():
    """ Test squid axon propagation.
    """
    moose.Neutral( '/library' )
    makeAxonProto()

    rdes = rd.rdesigneur(
        chanProto = [['make_HH_Na()', 'Na'], ['make_HH_K()', 'K']],
        cellProto = [['elec','axon']],
        chanDistrib = [
            ['Na', '#', 'Gbar', '1200' ],
            ['K', '#', 'Gbar', '360' ]],
        stimList = [['soma', '1', '.', 'inject', '(t>0.005 && t<0.2) * 2e-11' ]],
        plotList = [['soma', '1', '.', 'Vm', 'Membrane potential']],
        moogList = [['#', '1', '.', 'Vm', 'Vm (mV)']]
    )

    rdes.buildModel()
    moose.reinit()
    moose.start(0.04)
    data = moose.wildcardFind('/##[TYPE=Table]')[0].vector[::10]
    assert np.allclose(data, expected), "Error is %s" % (data-expected)
    print('Add done')
    return data

if __name__ == '__main__':
    test_axon_propagation()



# Rdesigneur tutorial converted to a test.
import moose
print("[INFO ] MOOSE version=%s, loc=%s" % (moose.version(), moose.__file__))
import rdesigneur as rd

def test_simple():
    """First test.
    >>> test_simple() # doctest: +NORMALIZE_WHITESPACE
    Rdesigneur: Elec model has 1 compartments and 0 spines on 0 compartments.
    <BLANKLINE>
    [/model[0]/elec[0]/soma[0]]
    Cm              =7.853981633975e-09
    Em              =-0.0544
    Im              =1.3194689277024895e-08
    Ra              =7639437.268410473
    Rm              =424413.1773342278
    Vm              =-0.06
    className       =ZombieCompartment
    diameter        =0.0005
    dt              =0.0
    fieldIndex      =0
    idValue         =455
    index           =0
    initVm          =-0.065
    inject          =0.0
    length          =0.0005
    name            =soma
    numData         =1
    numField        =1
    path            =/model[0]/elec[0]/soma[0]
    tick            =-2
    x               =0.0005
    x0              =0.0
    y               =0.0
    y0              =0.0
    z               =0.0
    z0              =0.0
    <BLANKLINE>
    INCOMING:
    /model[0]/elec[0]/soma ('parentMsg',) <--- /model[0]/elec ('childOut',)                                         
    OUTGOING:
    """
    rdes = rd.rdesigneur()
    rdes.buildModel()
    moose.showfield(rdes.soma)
    moose.showmsg(rdes.soma)

if __name__ == '__main__':
    import doctest
    doctest.testmod()



# test_negative_value_flag.py ---
#
# Filename: test_negative_value_flag.py
# Description:
# Author: Upi Bhalla
# Maintainer: Dilawar Singh <dilawars@ncbs.res.in>
# Created: Sat Oct 04 12:14:15 2014 (+0530)

import moose
import numpy as np
import sys
import os

scriptDir = os.path.dirname(os.path.realpath(__file__))


def test_neg_value_flag():
    """
    This example illustrates loading, running, and saving a kinetic
    model defined in kkit format. It uses a default kkit model but
    you can specify another using the command line
        ``python filename runtime solver``.
    We use the gsl solver here.
    The model already defines a couple of plots and sets the runtime 20 secs.
    """

    global scriptDir

    solver = "gsl"  # Pick any of gsl, gssa, ee..
    mfile = os.path.join(scriptDir, '..', 'data', 'acc11.g')
    assert os.path.isfile(mfile)
    runtime = 1000.0
    modelId = moose.loadModel(mfile, 'model')
    moose.element('/model/kinetics/neuroNOS/nNOS.arg').concInit = 0.1
    moose.reinit()
    moose.start(runtime)

    # Display all plots.
    for x in moose.wildcardFind('/model/#graphs/conc#/#'):
        t = np.arange(0, x.vector.size, 1) * x.dt
        print(x.vector)
        if x.vector.size > 0:
            assert min(x.vector) >= 0.0, min(x.vector)
        assert x.vector.size in [0, 10001], x.vector.size

    ########################################################
    # Run it again with negative values allowed
    stoich = moose.element('/model/kinetics/stoich')
    print(moose.wildcardFind('/##[TYPE=Stoich]'))
    stoich.allowNegative = True
    moose.reinit()
    moose.start(runtime)
    oneValIsBelowZero = False
    allVals = []
    for x in moose.wildcardFind('/model/#graphs/conc#/#'):
        t = np.arange(0, x.vector.size, 1) * x.dt
        if x.vector.size > 0:
            allVals.append(min(x.vector))
            if min(x.vector) <= 0.0:
                oneValIsBelowZero = True

        if allVals:
            assert oneValIsBelowZero, "No value is negative: %s" % allVals
        assert x.vector.size in [0, 10001], x.vector.size

def main():
    test_neg_value_flag()

if __name__ == '__main__':
    main()



# Na channel taken from mod files of Migliore2018: na3n.mod
# Problems: q10 same for both X and Y gates.

import numpy as np
import moose

SOMA_A = 3.14e-8
F = 96485.3329
R = 8.314
celsius = 32
dt = 0.05e-3
ENa = 0.092
EK = -0.100
Eh = -0.030
ECa = 0.140
Em = -0.065

Vmin = -0.100
Vmax = 0.100
Vdivs = 3000
v = np.linspace(Vmin,Vmax, Vdivs)
Camin = 1e-12
Camax = 1
Cadivs = 400
ca = np.linspace(Camin,Camax, Cadivs)

# original
Na_actA = -138.9
Na_actB = -5.34
Na_actC = 2.60187360e-05
Na_actD = -7.05694169e+01
Na_actE = 2.64467199e-02
Na_actF = -1.02623323e+02
Na_actG = 1.73923802e-05

Na_inactB = 12.5
Na_inactC = 2.004014672869906e-09
Na_inactD = -360.599
Na_inactE = 1.866086164497956e-09
Na_inactF = -454.5
Na_inactG = 0.00047


def Na_Chan(name):
    Na = moose.HHChannel( '/library/' + name )
    Na.Ek = ENa
    Na.Gbar = 300.0*SOMA_A
    Na.Gk = 0.0
    Na.Xpower = 3.0
    Na.Ypower = 1
    Na.Zpower = 0

    Na_inactA = 250

    def trap0(v,th,a,q):
        if np.abs(v*1e3-th) > 1e-6:
            return a * (v*1e3 - th) / (1 - np.exp(-(v*1e3 - th)/q))
        else:
            return a * q

    minf = 1/(1+np.exp(Na_actA*v+Na_actB))
    mtau=Na_actC*np.exp(Na_actD*v)/(1+Na_actE*np.exp(Na_actF*v)) + Na_actG
    hinf = 1/(1+np.exp(Na_inactA*v+Na_inactB))
    htau=Na_inactC*np.exp(Na_inactD*v)/(1+Na_inactE*np.exp(Na_inactF*v)) + Na_inactG

    xgate = moose.element( Na.path + '/gateX' )
    xgate.min = Vmin
    xgate.max = Vmax
    xgate.divs = Vdivs
    xgate.tableA = minf/mtau
    xgate.tableB = 1.0/mtau

    ygate = moose.element( Na.path + '/gateY' )
    ygate.min = Vmin
    ygate.max = Vmax
    ygate.divs = Vdivs
    ygate.tableA = hinf/htau
    ygate.tableB = 1.0/htau

    return Na



# -*- coding: utf-8 -*-
# This file is part of MOOSE simulator: http://moose.ncbs.res.in.

# MOOSE is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# MOOSE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with MOOSE.  If not, see <http://www.gnu.org/licenses/>.


"""moose_constants.py:

Last modified: Sat Jan 18, 2014  05:01PM

"""

__author__           = "Dilawar Singh"
__license__          = "GNU GPL"
__maintainer__       = "Dilawar Singh"
__email__            = "dilawars@ncbs.res.in"

## for Ca Pool
# FARADAY = 96154.0
# Coulombs
# from cadecay.mod : 1/(2*96154.0) = 5.2e-6 which is the Book of Genesis / readcell value
FARADAY = 96485.3415 # Coulombs/mol # from Wikipedia

## Table step_mode
# table acts as lookup - default mode
TAB_IO = 0

# table outputs value until it reaches the end and then stays at the last value
TAB_ONCE = 2

# table acts as a buffer: succesive entries at each time step
TAB_BUF = 3

# table acts as a buffer for spike times. Threshold stored in the pymoose 'stepSize' field.
TAB_SPIKE = 4

## Table fill modes
BSplineFill = 0 # B-spline fill (default)
CSplineFill = 1 # C_Spline fill (not yet implemented)
LinearFill = 2 # Linear fill

## clock 0 is for init & hsolve
## The ee method uses clocks 1, 2.
## hsolve & ee use clock 3 for Ca/ion pools.
## clocks 4 and 5 are for biochemical simulations.
## clock 6 for lookup tables, clock 7 for stimuli
## clocks 8 and 9 for tables for plots.
INITCLOCK = 0
ELECCLOCK = 1
CHANCLOCK = 2
POOLCLOCK = 3
LOOKUPCLOCK = 6
STIMCLOCK = 7
PLOTCLOCK = 8



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

import moose.utils as mu

from .NeuroML import NeuroML, loadNeuroML_L123
from .NetworkML import NetworkML
from .MorphML import MorphML
from .ChannelML import ChannelML



# -*- coding: utf-8 -*-

## for Ca Pool
#FARADAY = 96154.0 # Coulombs # from cadecay.mod : 1/(2*96154.0) = 5.2e-6 which is the Book of Genesis / readcell value
FARADAY = 96485.3415 # Coulombs/mol # from Wikipedia

## Table step_mode
TAB_IO=0 # table acts as lookup - default mode
TAB_ONCE=2 # table outputs value until it reaches the end and then stays at the last value
TAB_BUF=3 # table acts as a buffer: succesive entries at each time step
TAB_SPIKE=4 # table acts as a buffer for spike times. Threshold stored in the pymoose 'stepSize' field.

## Table fill modes
BSplineFill = 0 # B-spline fill (default)
CSplineFill = 1 # C_Spline fill (not yet implemented)
LinearFill = 2 # Linear fill

## The hsolve and ee methods use clock 1.
## hsolve further uses clock 2 for mg_block, nmdachan and others.
## keep clock 3 for plots.
PLOTCLOCK = 3



# -*- coding: utf-8 -*-
# test_neurom2.py, modified from run_cell.py
# Maintainer: P Gleeson, Dilawar Singh
# This test is not robust.
# Code:

from __future__ import absolute_import, print_function, division

# check if neuroml working properly.
# NOTE: This script does not work with python3
# See https://github.com/NeuroML/NeuroML2/issues/116 . If this bug is fixed then
# remove this code block.

import moose
import moose.utils as mu
import os
import numpy as np

SCRIPT_DIR = os.path.dirname( os.path.realpath( __file__ ) )


def test_nml2( nogui = True ):
    global SCRIPT_DIR
    filename = os.path.join(SCRIPT_DIR, 'test_files/passiveCell.nml' )
    mu.info('Loading: %s' % filename )
    nml = moose.readNML2( filename )
    if not nml:
        mu.warn( "Failed to parse NML2 file" )
        return

    assert nml, "Expecting NML2 object"
    msoma = nml.getComp(nml.doc.networks[0].populations[0].id,0,0)
    data = moose.Neutral('/data')
    pg = nml.getInput('pulseGen1')

    inj = moose.Table('%s/pulse' % (data.path))
    moose.connect(inj, 'requestOut', pg, 'getOutputValue')

    vm = moose.Table('%s/Vm' % (data.path))
    moose.connect(vm, 'requestOut', msoma, 'getVm')

    simtime = 150e-3
    moose.reinit()
    moose.start(simtime)
    print("Finished simulation!")
    yvec = vm.vector
    injvec = inj.vector * 1e12
    m1, u1 = np.mean( yvec ), np.std( yvec )
    m2, u2 = np.mean( injvec ), np.std( injvec )
    assert np.isclose( m1, -0.0456943 ), m1
    assert np.isclose( u1, 0.0121968 ), u1
    assert np.isclose( m2, 26.64890 ), m2
    assert np.isclose( u2, 37.70607574 ), u2

if __name__ == '__main__':
    test_nml2()



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
All SI units
## Plot the firing rate vs current injection curve for a Cerebellar Granule Cell neuron

## Author: Aditya Gilra
## Creation Date: 2012-07-12
## Modification Date: 2012-07-12

                    Wednesday 15 July 2015 09:46:36 AM IST
                    Added unittest
                    Modified for testing with ctest.
"""
from __future__ import print_function

import os
os.environ['NUMPTHREADS'] = '1'

import moose
from moose.neuroml.NeuroML import NeuroML

import numpy as np
import unittest

soma_ = None
cellSpikeTable_ = None

def loadModel(filename):
    global soma_, cellSpikeTable_
    neuromlR = NeuroML()
    neuromlR.readNeuroMLFromFile(filename)
    libcell = moose.Neuron('/library/CA1soma')
    CA1Cellid = moose.copy(libcell,moose.Neutral('/cells'),'CA1')
    CA1Cell = moose.Neuron(CA1Cellid)

    spikeGen = moose.SpikeGen(CA1Cell.path+'/spikeGen')
    spikeGen.threshold = -30e-3 # V
    soma_ = moose.Compartment(CA1Cell.path+'/Seg0_soma_0_0')
    soma_.inject = 0 # by default the cell has a current injection
    moose.connect(soma_,'VmOut',spikeGen,'Vm')
    table_path = moose.Neutral(CA1Cell.path+'/data').path
    cellSpikeTable_ = moose.Table(table_path+'/spikesTable')
    moose.connect(spikeGen,'spikeOut', cellSpikeTable_,'input')

def applyCurrent(currenti):
    global soma_, cellSpikeTable_
    moose.reinit()
    soma_.inject = currenti
    moose.start(1.0)
    spikesList = np.array(cellSpikeTable_.vector)
    if len(spikesList)>0:
        spikesList = spikesList[np.where(spikesList>0.0)[0]]
        spikesNow = len(spikesList)
    else:
        spikesNow = 0.0
    print("For injected current {0}, no of spikes in {1} second: {2}".format(
        currenti, 1.0, spikesNow )
        )
    return spikesNow

def ca1_main( currenti):
    return applyCurrent(currenti)



# -*- coding: utf-8 -*-
r'''
*******************************************************************
 * File:            writeSBML.py
 * Description:
 * Author:          HarshaRani
 * E-mail:          hrani@ncbs.res.in
 ********************************************************************/
/**********************************************************************
** This program is part of 'MOOSE', the
** Messaging Object Oriented Simulation Environment,
** also known as GENESIS 3 base code.
**           copyright (C) 2003-2017 Upinder S. Bhalla. and NCBS
Created : Friday May 27 12:19:00 2016(+0530)
Version
Last-Updated: Tue 08 Sep 02:22:10 2020(+0530)
          By: HarshaRani
**********************************************************************/
/****************************
2020
Sep 08: replaced getId->id, getId().value-> idValue,getDataIndex-dataIndex, isinstance is replaced with isA these changes are with respect to
        new python-binding c38580789d8fb65
Mar 20: Channel Annotation is cleaned up
Jan 08: added function to write Concchannel in form of MMenz
        Km in the kinetic law for MMenz is written to the power based on the number of substrate
2019
July 18: added a call for autolayout, this was required for cspace model while trying to write from cmd line
         while writting file, the filepath is checked
         now even for cplxpool's x and y co-ordinates,diff and motor constant are added
Jan 29: getColor are taken from chemConnectUtil, group's width and height are written
2018
Dec 07: using fixXreac's restoreXreacs function to remove xfer
Dec 03: add diff and motor constants to pool
Nov 30: group id is changed from name to moose_id and group.name is added along with annotation for group listing
Nov 22: searched for _xfer_ instead of xfer
Nov 12: xfer cross compartment molecules are not written to SBML instead written the original molecule also for connecting Reaction and Enzyme 
Nov 06: All the Mesh Cyl,Cube,Neuro,Endo Mesh's can be written into SBML format with annotation field where Meshtype
        numDiffCompts,isMembraneBound and surround are written out.
        For EndoMesh check made to see surround is specified
Oct 20: EndoMesh added to SBML
Oct 16: CylMesh's comparment volume is written, but zeroth volex details are populated 
Oct 13: CylMesh are written to SBML with annotation field and only zeroth element/voxel (incase of cylMesh) of moose object is written
Oct 1 : corrected the spell of CyclMesh-->CylMesh, negating the yaxis for kkit is removed 
Apr 30: indentation corrected while writting annotation for enzymecomplex
Jan 6 : for Product_formation_, k3 units depends on noofSub, prd was passed which is fixed
2017
Dec 15: If model path exists is checked
        Enz cplx is written only if enz,cplx,sub, prd exist, a clean check is made
        Checked if function exist inputs and also if expr is exist, then written.
Aug 8 : removed "findCompartment" function to chemConnectUtil and imported the function from the same file
        convertSpecialChar for setId and convertSpecialCharshot for setName.
        specialChar like /,\,[,],space are not allowed as moose doesn't take
Aug 3 : Added recalculatecoordinates,cleanup in groupName
'''

import sys
import re
import os
import moose
from moose.SBML.validation import validateModel
from moose.chemUtil.chemConnectUtil import xyPosition,mooseIsInstance,findCompartment,getColor,setupItem,setupMeshObj
from moose.chemUtil.graphUtils import *
from moose.fixXreacs import restoreXreacs
import numpy as np

foundLibSBML_ = False
try:
    from libsbml import *
    foundLibSBML_ = True
except Exception as e:
    pass

def checkPath( dirName):
    path = dirName
    if (dirName == "~" or not dirName ):
        if dirName:
            dirName = os.path.expanduser(dirName)
        else:
            dirName = os.getcwd()
    
    if os.access(dirName, os.W_OK) is not True:
        dirName = os.getcwd()
        print(path +" not writable, writting to "+dirName+ " directory")
        return dirName
    else:
        return dirName

def mooseWriteSBML(modelpath, filename, sceneitems={}):
    global foundLibSBML_
    msg = " "
    if not foundLibSBML_:
        print('No python-libsbml found.'
            '\nThis module can be installed by following command in terminal:'
            '\n\t easy_install python-libsbml or'
            '\n\t apt-get install python-libsbml'
            )
        return -2, "Could not save the model in to SBML file. \nThis module can be installed by following command in terminal: \n\t easy_install python-libsbml or \n\t apt-get install python-libsbml",''

    #sbmlDoc = SBMLDocument(3, 1)
    filepath, filenameExt = os.path.split(filename)
    filepath = checkPath(filepath)
    if filenameExt.find('.') != -1:
        filename = filenameExt[:filenameExt.find('.')]
    else:
        filename = filenameExt
    
    # validatemodel
    sbmlOk = False
    global spe_constTrue
    spe_constTrue = []
    global nameList_
    nameList_ = []

    xcord,ycord = [],[]
    if not moose.exists(modelpath):
        return False, "Path doesn't exist"
    elif moose.exists(modelpath):
        moose.fixXreacs.restoreXreacs(modelpath)
        checkCompt = moose.wildcardFind(modelpath+'/##[0][ISA=ChemCompt]')
        
        mObj = moose.wildcardFind(moose.element(modelpath).path+'/##[0][ISA=PoolBase]'+','+
                                  moose.element(modelpath).path+'/##[0][ISA=ReacBase]'+','+
                                  moose.element(modelpath).path+'/##[0][ISA=EnzBase]'+','+
                                  moose.element(modelpath).path+'/##[0][ISA=StimulusTable]')
        for p in mObj:
            if not moose.element(p.parent).isA("CplxEnzBase"):
                if moose.exists(p.path+'/info'):
                    xcord.append(moose.element(p.path+'/info').x)
                    ycord.append(moose.element(p.path+'/info').y)
                    getColor(moose.element(p.path+'/info').path)
        recalculatecoordinates(modelpath,mObj,xcord,ycord)
    positionInfoexist = False

    xmlns = SBMLNamespaces(3, 1)
    xmlns.addNamespace("http://www.w3.org/1999/xhtml", "xhtml")
    xmlns.addNamespace("http://www.moose.ncbs.res.in", "moose")
    xmlns.addNamespace("http://www.sbml.org/sbml/level3/version1/groups/version1", "groups")
    sbmlDoc = SBMLDocument(xmlns)
    sbmlDoc.setPackageRequired("groups",bool(0))

    cremodel_ = sbmlDoc.createModel()
    cremodel_.setId(filename)
    cremodel_.setTimeUnits("time")
    cremodel_.setExtentUnits("substance")
    cremodel_.setSubstanceUnits("substance")
    cremodel_.setVolumeUnits("volume")
    cremodel_.setAreaUnits("area")
    cremodel_.setLengthUnits("length")
    neutralNotes = ""

    specieslist = moose.wildcardFind(modelpath + '/##[0][ISA=PoolBase]')
    
    if specieslist:
        neutralPath = getGroupinfo(specieslist[0])
        if moose.exists(neutralPath.path + '/info'):
            neutralInfo = moose.element(neutralPath.path + '/info')
            neutralNotes = neutralInfo.notes
        if neutralNotes != "":
            cleanNotes = convertNotesSpecialChar(neutralNotes)
            notesString = "<body xmlns=\"http://www.w3.org/1999/xhtml\">\n \t \t" + \
                neutralNotes + "\n\t </body>"
            cremodel_.setNotes(notesString)

    srcdesConnection = {}

    writeUnits(cremodel_)
    
    modelAnno = writeSimulationAnnotation(modelpath)
    
    if modelAnno:
        cremodel_.setAnnotation(modelAnno)
    groupInfo = {}
    reacGroup = {}
    compterrors =""
    compartexist, groupInfo,compterrors = writeCompt(modelpath, cremodel_)
    
    if compartexist == True:
        species = writeSpecies( modelpath,cremodel_,sbmlDoc,sceneitems,groupInfo)
        if species:
            writeFunc(modelpath, cremodel_)
        
        writeChannel(modelpath,cremodel_,sceneitems,groupInfo)

        writeReac(modelpath, cremodel_, sceneitems,groupInfo)
        
        writeEnz(modelpath, cremodel_, sceneitems,groupInfo)
        
        if groupInfo:
            for key,value in groupInfo.items():
                mplugin = cremodel_.getPlugin("groups")
                group = mplugin.createGroup()
                name = str(idBeginWith(moose.element(key).name))
                moosegrpId = name +"_" + str(moose.element(key).idValue) + "_" + str(moose.element(key).dataIndex)+"_"
                group.setId(moosegrpId) 
                group.setName(name)

                group.setKind("collection")
                if moose.exists(key.path+'/info'):
                    ginfo = moose.element(key.path+'/info')
                else:
                    ginfo = moose.Annotator(key.path+'/info')
                
                groupCompartment = findCompartment(key)
                
                grpAnno = "<moose:GroupAnnotation>"
                grpAnno = grpAnno + "<moose:Compartment>" + groupCompartment.name +"_"+ str(moose.element(groupCompartment).idValue) +"_"+ str(moose.element(groupCompartment).dataIndex)+"_"+"</moose:Compartment>\n"
                #grpAnno = grpAnno + "<moose:Compartment>" + groupCompartment.name + "_" + str(moose.element(groupCompartment).idValue) + "_" + str(moose.element(groupCompartment).dataIndex) + "_"+ "</moose:Compartment>\n"
                if moose.element(key.parent).className == "Neutral":

                    grpAnno = grpAnno + "<moose:Group>" + key.parent.name + "</moose:Group>\n"
                    grpparent = key.parent.name + "_" + str(moose.element(key.parent).idValue) + "_" + str(moose.element(key.parent).dataIndex) + "_"
                    grpAnno = grpAnno + "<moose:Parent>" + grpparent + "</moose:Parent>\n"
                else:
                    grpparent = groupCompartment.name + "_" + str(moose.element(groupCompartment).idValue) + "_" + str(moose.element(groupCompartment).dataIndex) + "_"
                    grpAnno = grpAnno + "<moose:Parent>" + grpparent + "</moose:Parent>\n"
                
                if moose.exists(key.path+'/info'):
                    ginfo = moose.element(key.path+'/info')
                    textColor,color = getColor(ginfo)
                    if ginfo.height and ginfo.width:
                        grpAnno = grpAnno + "<moose:x>" + str(ginfo.x) + "</moose:x>\n"
                        grpAnno = grpAnno + "<moose:y>" + str(ginfo.y) + "</moose:y>\n"
                        grpAnno = grpAnno + "<moose:width>" + str(ginfo.width) + "</moose:width>\n"
                        grpAnno = grpAnno + "<moose:height>" + str(ginfo.height) + "</moose:height>\n"
                    if ginfo.color:
                        grpAnno = grpAnno + "<moose:bgColor>" + color + "</moose:bgColor>\n"
                    if ginfo.notes:
                        grpAnno = grpAnno + "<moose:Notes>" + ginfo.notes + "</moose:Notes>\n"
                grpAnno = grpAnno + "</moose:GroupAnnotation>"
                group.setAnnotation(grpAnno)

                for values in value:
                    member = group.createMember()
                    member.setIdRef(values)
        
        consistencyMessages = ""
        SBMLok = validateModel(sbmlDoc)
        if (SBMLok):
            if filepath != [r" ", r"\/", r"/"]:
                writeTofile = filepath + "/" + filename + '.xml'
            else:
                writeTofile = filename+'.xml'
            #writeTofile = filepath + "/" + filename + '.xml'
            writeSBMLToFile(sbmlDoc, writeTofile)
            return True, consistencyMessages, writeTofile
        
        else:
            #cerr << "Errors encountered " << endl
            consistencyMessages = "Errors encountered"
            return -1, consistencyMessages
    else:
        if compterrors:
            return False, compterrors
        else:
            return False,"Atleast one compartment should exist to write SBML"

def writeChannel(modelpath, cremodel_, sceneitems,groupInfo):
    for chan in moose.wildcardFind(modelpath + '/##[0][ISA=ConcChan]'):
        chanannoexist = False
        chanGpnCorCol = " "
        cleanChanname = convertSpecialChar(chan.name)
        compt = ""
        notesE = ""
        groupName = moose.element("/")
        if not moose.exists(chan.path+'/info'):
            moose.Annotator(chan.path+'/info')
        if moose.exists(chan.path + '/info'):
            Anno = moose.Annotator(chan.path + '/info')
            notesE = Anno.notes
            element = moose.element(chan)
            ele = getGroupinfo(element)
            ele = findGroup_compt(element)
            chanAnno = " "
            #if ele.className == "Neutral" or sceneitems or Anno.x or Anno.y:
            #        chanannoexist = True
            chanannoexist = True
            if chanannoexist:
                chanAnno = "<moose:ModelAnnotation>\n"
                if ele.className == "Neutral":
                    groupName = ele
                if sceneitems:
                    #Saved from GUI, then scene co-ordinates are passed
                    chanGpnCorCol = "<moose:xCord>" + \
                            str(sceneitems[chan]['x']) + "</moose:xCord>\n" + \
                            "<moose:yCord>" + \
                            str(sceneitems[chan]['y'])+ "</moose:yCord>\n"
                else:
                    #Saved from cmdline,genesis coordinates are kept as its
                    # SBML, cspace, python, then auto-coordinates are done
                    #and coordinates are updated in moose Annotation field
                    chanGpnCorCol = "<moose:xCord>" + \
                            str(Anno.x) + "</moose:xCord>\n" + \
                            "<moose:yCord>" + \
                            str(Anno.y)+ "</moose:yCord>\n"
                chanGpnCorCol = chanGpnCorCol+"<moose:Permeability>"+str(chan.permeability)+"</moose:Permeability>\n"
        chanSub = chan.neighbors["in"]
        chanPrd = chan.neighbors["out"]
        if (len(chanSub) != 0 and len(chanPrd) != 0):
            chanCompt = findCompartment(chan)
            if not moose.element(chanCompt).isA("ChemCompt"):
                return -2
            else:
                compt = chanCompt.name + "_" + \
                    str(chanCompt.idValue) + "_" + \
                    str(chanCompt.dataIndex) + "_"
  
            channel = cremodel_.createReaction()
            if notesE != "":
                cleanNotesE = convertNotesSpecialChar(notesE)
                notesStringE = "<body xmlns=\"http://www.w3.org/1999/xhtml\">\n \t \t" + \
                    cleanNotesE + "\n\t </body>"
                channel.setNotes(notesStringE)

            chansetId = str(idBeginWith(cleanChanname +
                                         "_" +
                                         str(chan.idValue) +
                                         "_" +
                                         str(chan.dataIndex) +
                                         "_"))
            channel.setId(chansetId)
            if groupName != moose.element('/'):
                if groupName not in groupInfo:
                    groupInfo[groupName]=[chansetId]
                else:
                    groupInfo[groupName].append(chansetId)

            channel.setName(str(idBeginWith(convertSpecialCharshot(chan.name))))
            channel.setReversible(True)
            channel.setFast(False)
            chanAnno = "<moose:ModelAnnotation>\n <moose:Channel>ConcChannel</moose:Channel>\n" + \
                    chanGpnCorCol + "</moose:ModelAnnotation>"
            channel.setAnnotation(chanAnno)

            noofSub, sRateLawS = getSubprd(cremodel_, False, "sub", chanSub)
            # Modifier
            chanMod = chan.neighbors["setNumChan"]
            noofMod, sRateLawM = getSubprd(cremodel_, False, "enz", chanMod)
            
            noofPrd, sRateLawP = getSubprd(cremodel_, False, "prd", chanPrd)
            
            kl = channel.createKineticLaw()
            
            fRate_law = compt + " * ( Permeability) * " + sRateLawM + " * (" + sRateLawS+ " - " + sRateLawP  +")"
            kl.setFormula(fRate_law)
            kl.setNotes(
                "<body xmlns=\"http://www.w3.org/1999/xhtml\">\n\t\t" +
                fRate_law +
                "\n \t </body>")
            
            channelUnit = permeablUnit(cremodel_)
            printParameters(kl, "Permeability", chan.permeability, channelUnit)    
def writeEnz(modelpath, cremodel_, sceneitems,groupInfo):
    for enz in moose.wildcardFind(modelpath + '/##[0][ISA=EnzBase]'):
        enzannoexist = False
        enzGpnCorCol = " "
        cleanEnzname = convertSpecialChar(enz.name)
        enzSubt = ()
        compt = ""
        notesE = ""
        groupName = moose.element("/")

        if moose.exists(enz.path + '/info'):
            groupName = moose.element("/")
            Anno = moose.Annotator(enz.path + '/info')
            notesE = Anno.notes
            element = moose.element(enz)
            ele = getGroupinfo(element)
            ele = findGroup_compt(element)
            if ele.className == "Neutral" or sceneitems or Anno.x or Anno.y:
                    enzannoexist = True
            if enzannoexist:
                enzAnno = "<moose:ModelAnnotation>\n"
                if ele.className == "Neutral":
                    groupName = ele
                    #enzGpnCorCol =  "<moose:Group>" + ele.name + "</moose:Group>\n"
                    # if ele.name not in groupInfo:
                    #         groupInfo[ele.name]=[setId]
                    #     else:
                    #         groupInfo[ele.name].append(setId)
                if sceneitems:
                    #Saved from GUI, then scene co-ordinates are passed
                    enzGpnCorCol = enzGpnCorCol + "<moose:xCord>" + \
                            str(sceneitems[enz]['x']) + "</moose:xCord>\n" + \
                            "<moose:yCord>" + \
                            str(sceneitems[enz]['y'])+ "</moose:yCord>\n"
                else:
                    #Saved from cmdline,genesis coordinates are kept as its
                    # SBML, cspace, python, then auto-coordinates are done
                    #and coordinates are updated in moose Annotation field
                    enzGpnCorCol = enzGpnCorCol + "<moose:xCord>" + \
                            str(Anno.x) + "</moose:xCord>\n" + \
                            "<moose:yCord>" + \
                            str(Anno.y)+ "</moose:yCord>\n"
                if Anno.color:
                    enzGpnCorCol = enzGpnCorCol + "<moose:bgColor>" + Anno.color + "</moose:bgColor>\n"
                if Anno.textColor:
                    enzGpnCorCol = enzGpnCorCol + "<moose:textColor>" + \
                        Anno.textColor + "</moose:textColor>\n"
        if (enz.className == "Enz" or enz.className == "ZombieEnz"):
            # Enz cplx is written into 2 reaction S+E-> SE*, SE* -> E+P
            foundEnzymeComplex = False
            comptVec = findCompartment(moose.element(enz))
            if not moose.element(comptVec).isA("ChemCompt"):
                return -2
            else:
                compt = comptVec.name + "_" + \
                    str(comptVec.idValue) + "_" + \
                    str(comptVec.dataIndex) + "_"
            #Writting out S+E -> SE*
            enzsetId = str(idBeginWith(cleanEnzname +
                                         "_" + str(enz.idValue) +
                                         "_" + str(enz.dataIndex) +
                                         "_" + "Complex_formation_"))
            #Finding Enzyme parent (E), each enzyme should just have one parent, multiple parent not right!
            secplxerror = ""
            enzSubt = ()
            enzOut = enz.neighbors["enzOut"]
            if not enzOut:
                secplxerror = "enzyme parent missing "
            else:
                enzAnno = "<moose:EnzymaticReaction>\n"
                listofname(enzOut, True)
                enzSubt = enzOut
                if len(nameList_) != 1:
                    secplxerror = secplxerror +" multiple enzyme parent present"
                else:
                    for i in range(0, len(nameList_)):
                        enzAnno = enzAnno + "<moose:enzyme>" + \
                            (str(idBeginWith(convertSpecialChar(
                                nameList_[i])))) + "</moose:enzyme>\n"
                    #Finding Substrate, (S)
                    enzSub = enz.neighbors["sub"]
                    if not enzSub:
                        secplxerror = secplxerror + " substrate missing"
                    else:
                        listofname(enzSub, True)
                        enzSubt += enzSub
                        for i in range(0, len(nameList_)):
                            enzAnno = enzAnno + "<moose:substrates>" + \
                            nameList_[i] + "</moose:substrates>\n"

                        #Finding product, which in this case the cplx (SE*)
                        enzPrd = enz.neighbors["cplxDest"]
                        noofPrd = len(enzPrd)
                        if not enzPrd:
                            secplxerror = secplxerror + " enzymecplx missing which act as product "                            
                        else:
                            listofname(enzPrd, True)
                            for i in range(0, len(nameList_)):
                                enzAnno = enzAnno + "<moose:product>" + \
                                    nameList_[i] + "</moose:product>\n"
                            foundEnzymeComplex = True
            if foundEnzymeComplex:
                # Only if S+E->SE* found, reaction is created
                enzyme = cremodel_.createReaction()
                if notesE != "":
                    cleanNotesE = convertNotesSpecialChar(notesE)
                    notesStringE = "<body xmlns=\"http://www.w3.org/1999/xhtml\">\n \t \t" + \
                                    cleanNotesE + "\n\t </body>"
                    enzyme.setNotes(notesStringE)
                
                enzyme.setId(enzsetId)
                enzyme.setName(str(idBeginWith(convertSpecialCharshot(enz.name))))
                enzyme.setFast(False)
                enzyme.setReversible(True)
                k1 = enz.concK1
                k2 = enz.k2
                k3 = enz.k3
                
                enzAnno = enzAnno + "<moose:groupName>" + cleanEnzname + "_" + \
                str(enz.idValue) + "_" + \
                str(enz.dataIndex) + "_" + "</moose:groupName>\n"
                enzAnno = enzAnno + "<moose:stage>1</moose:stage>\n"
                if enzannoexist:
                    enzAnno = enzAnno + enzGpnCorCol
                enzAnno = enzAnno + "</moose:EnzymaticReaction>"
                enzyme.setAnnotation(enzAnno)
                if enzSubt:
                    rate_law = "k1"
                    noofSub, sRateLaw = getSubprd(cremodel_, True, "sub", enzSubt)
                    rate_law = compt + " * ( " + rate_law + " * " + sRateLaw
                if enzPrd:
                    noofPrd, sRateLaw = getSubprd(cremodel_, True, "prd", enzPrd)
                    rate_law = rate_law + " - " + " k2 " + ' * ' + sRateLaw +" )"
                kl = enzyme.createKineticLaw()
                kl.setFormula(rate_law)
                kl.setNotes("<body xmlns=\"http://www.w3.org/1999/xhtml\">\n\t\t" + \
                            rate_law + "\n \t </body>")
                unit = parmUnit(noofSub - 1, cremodel_)
                printParameters(kl, "k1", k1, unit)
                punit = parmUnit(noofPrd - 1, cremodel_)
                printParameters(kl, "k2", k2, punit)
                if groupName != moose.element('/'):
                    if groupName not in groupInfo:
                        #groupInfo[groupName]=[enzsetId]
                        groupInfo[groupName]=[nameList_[0]]
                    else:
                        #groupInfo[groupName].append(enzsetId)
                        groupInfo[groupName].append(nameList_[0])
            else:
                if secplxerror:
                    print ("\'"+enz.parent.name+ "/"+enz.name+"\' this enzyme is not written to file because,"+ secplxerror)
            
            #Here SE* -> E+ P
            foundEnzymeEP = False
            enzsetIdP = str(idBeginWith(cleanEnzname +
                                        "_" + str(enz.idValue) +
                                        "_" + str(enz.dataIndex) +
                                        "_" + "Product_formation_"))
            cplxeperror = ""
            enzAnno2 = ""
            #enzSubt = ""
            enzOut = enz.neighbors["enzOut"]
            if not enzOut:
                cplxepeerror = "enzyme parent missing "
            else:
                enzAnno2 = "<moose:EnzymaticReaction>\n"
                listofname(enzOut, True)
                #enzSubt = enzOut
                if len(nameList_) != 1:
                    cplxeperror = cplxeperror +" multiple enzyme parent present"
                else:
                    # for i in range(0, len(nameList_)):
                    #     enzEnz = "<moose:enzyme>" + \
                    #             (str(idBeginWith(convertSpecialChar(
                    #             nameList_[i])))) + "</moose:enzyme>\n"
                    #Finding Substrate, which is (SE*)
                    enzSub = enz.neighbors["cplxDest"]
                    noofSub = len(enzSub)
                    listofname(enzSub, True)
                    if not enzSub:
                        cplxeperror = cplxeperror +"complex missing which act as substrate "
                    else:
                        for i in range(0, len(nameList_)):
                            enzAnno2 =enzAnno2 +"<moose:complex>" + \
                                    nameList_[i] + "</moose:complex>\n"
                        listofname(enzOut, True)
                        for i in range(0, len(nameList_)):
                            enzAnno2 = enzAnno2 +"<moose:enzyme>" + \
                                (str(idBeginWith(convertSpecialChar(
                                nameList_[i])))) + "</moose:enzyme>\n"

                        enzPrd = enz.neighbors["prd"]
                        noofPrd = len(enzPrd)
                        if not enzPrd:
                            cplxeperror = cplxeperror + "product missing "
                        else:
                            listofname(enzPrd,True)
                            for i in range(0, len(nameList_)):
                                enzAnno2 = enzAnno2 + "<moose:product>" + \
                                            nameList_[i] + "</moose:product>\n"
                        enzAnno2 += "<moose:groupName>" + cleanEnzname + "_" + \
                            str(enz.idValue) + "_" + \
                            str(enz.dataIndex) + "_" + "</moose:groupName>\n"
                        enzAnno2 += "<moose:stage>2</moose:stage> \n"
                        if enzannoexist:
                            enzAnno2 = enzAnno2 + enzGpnCorCol
                        enzAnno2 += "</moose:EnzymaticReaction>"

                        foundEnzymeEP = True
            if foundEnzymeEP:
                enzyme = cremodel_.createReaction()
                enzyme.setId(enzsetIdP)
                enzyme.setName(str(idBeginWith(convertSpecialCharshot(enz.name))))
                enzyme.setFast(False)
                enzyme.setReversible(False)
                enzyme.setAnnotation(enzAnno2)
                noofSub, sRateLaw = getSubprd(cremodel_, True, "sub", enzSub)
                enzprdt = ()
                enzPrdt = enzPrd+enzOut
                noofprd, sRateLaw2 = getSubprd(cremodel_, True, "prd", enzPrdt)
                enzrate_law = compt + " * k3" + '*' + sRateLaw
                kl = enzyme.createKineticLaw()
                kl.setFormula(enzrate_law)
                kl.setNotes(
                        "<body xmlns=\"http://www.w3.org/1999/xhtml\">\n\t\t" +
                        enzrate_law +
                        "\n \t </body>")
                unit = parmUnit(noofSub - 1, cremodel_)
                printParameters(kl, "k3", k3, unit)
                if groupName != moose.element('/'):
                    if groupName not in groupInfo:
                        #groupInfo[groupName]=[nameList_[0]]
                        pass
                    else:
                        #groupInfo[groupName].append(nameList_[0])
                        pass
            else:
                print (cplxeperror)
        elif(enz.className == "MMenz" or enz.className == "ZombieMMenz"):

            enzSub = enz.neighbors["sub"]
            enzPrd = enz.neighbors["prd"]
            if (len(enzSub) != 0 and len(enzPrd) != 0):
                enzCompt = findCompartment(enz)
                if not moose.element(enzCompt).isA("ChemCompt"):
                    return -2
                else:
                    compt = enzCompt.name + "_" + \
                        str(enzCompt.idValue) + "_" + \
                        str(enzCompt.dataIndex) + "_"
                enzyme = cremodel_.createReaction()
                enzAnno = " "
                if notesE != "":
                    cleanNotesE = convertNotesSpecialChar(notesE)
                    notesStringE = "<body xmlns=\"http://www.w3.org/1999/xhtml\">\n \t \t" + \
                        cleanNotesE + "\n\t </body>"
                    enzyme.setNotes(notesStringE)
                mmenzsetId = str(idBeginWith(cleanEnzname +
                                             "_" +
                                             str(enz.idValue) +
                                             "_" +
                                             str(enz.dataIndex) +
                                             "_"))
                enzyme.setId(mmenzsetId)
                if groupName != moose.element('/'):
                    if groupName not in groupInfo:
                        groupInfo[groupName]=[mmenzsetId]
                    else:
                        groupInfo[groupName].append(mmenzsetId)
                enzyme.setName(str(idBeginWith(convertSpecialCharshot(enz.name))))
                #enzyme.setName(cleanEnzname)
                enzyme.setFast(False)
                enzyme.setReversible(True)
                if enzannoexist:
                    enzAnno = enzAnno + enzGpnCorCol
                    enzAnno = "<moose:EnzymaticReaction>\n" + \
                        enzGpnCorCol + "</moose:EnzymaticReaction>"
                    enzyme.setAnnotation(enzAnno)
                Km = enz.Km
                kcat = enz.kcat
                enzSub = enz.neighbors["sub"]
                noofSub, sRateLawS = getSubprd(cremodel_, False, "sub", enzSub)
                #sRate_law << rate_law.str();
                # Modifier
                enzMod = enz.neighbors["enzDest"]
                noofMod, sRateLawM = getSubprd(cremodel_, False, "enz", enzMod)
                enzPrd = enz.neighbors["prd"]
                noofPrd, sRateLawP = getSubprd(cremodel_, False, "prd", enzPrd)
                kl = enzyme.createKineticLaw()
                #rate_law = clean_name + "^" + str(count)
                fRate_law = compt + " * ( kcat * " + sRateLawS + " * " + sRateLawM + "/(Km"
                
                if len(enzSub) > 1:
                    fRate_law = fRate_law +"^" +str(len(enzSub))  
                    print("enzyme ",enzyme.name, "number of substrate is greater than 1, kinetics Law Km is written to the power of substrate assumed that km value is factored")
                fRate_law = fRate_law +" + " + sRateLawS + "))"

                kl.setFormula(fRate_law)
                kl.setNotes(
                    "<body xmlns=\"http://www.w3.org/1999/xhtml\">\n\t\t" +
                    fRate_law +
                    "\n \t </body>")
                KmUnit(cremodel_)
                printParameters(kl, "Km", Km, "mmole_per_litre")
                kcatUnit = parmUnit(0, cremodel_)
                printParameters(kl, "kcat", kcat, kcatUnit)


def printParameters(kl, k, kvalue, unit):
    para = kl.createParameter()
    para.setId(str(idBeginWith(k)))
    para.setValue(kvalue)
    para.setUnits(unit)

def permeablUnit(cremodel_):
    unit_stream = "litre_per_mmole_per_second"
    lud = cremodel_.getListOfUnitDefinitions()
    flag = False
    for i in range(0, len(lud)):
        ud = lud.get(i)
        if (ud.getId() == unit_stream):
            flag = True
            break
    if (not flag):
        unitdef = cremodel_.createUnitDefinition()
        unitdef.setId(unit_stream)
        
        unit = unitdef.createUnit()
        unit.setKind(UNIT_KIND_LITRE)
        unit.setExponent(1)
        unit.setMultiplier(1)
        unit.setScale(0)

        unit = unitdef.createUnit()
        unit.setKind(UNIT_KIND_MOLE)
        unit.setExponent(-1)
        unit.setMultiplier(1)
        unit.setScale(-3)
        
        unit = unitdef.createUnit()
        unit.setKind(UNIT_KIND_SECOND)
        unit.setExponent(-1)
        unit.setMultiplier(1)
        unit.setScale(0)
    return unit_stream

def KmUnit(cremodel_):
    unit_stream = "mmole_per_litre"
    lud = cremodel_.getListOfUnitDefinitions()
    flag = False
    for i in range(0, len(lud)):
        ud = lud.get(i)
        if (ud.getId() == unit_stream):
            flag = True
            break
    if (not flag):
        unitdef = cremodel_.createUnitDefinition()
        unitdef.setId(unit_stream)
        unit = unitdef.createUnit()
        unit.setKind(UNIT_KIND_LITRE)
        unit.setExponent(-1)
        unit.setMultiplier(1)
        unit.setScale(0)
        unit = unitdef.createUnit()
        unit.setKind(UNIT_KIND_MOLE)
        unit.setExponent(1)
        unit.setMultiplier(1)
        unit.setScale(-3)
    return unit_stream

def parmUnit(rct_order, cremodel_):
    order = rct_order
    if order == 0:
        unit_stream = "per_second"
    elif order == 1:
        unit_stream = "litre_per_mmole_per_second"
    elif order == 2:
        unit_stream = "sq_litre_per_mmole_sq_per_second"
    else:
        unit_stream = "litre_per_mmole_" + str(rct_order) + "_per_second"

    lud = cremodel_.getListOfUnitDefinitions()
    flag = False
    for i in range(0, len(lud)):
        ud = lud.get(i)
        if (ud.getId() == unit_stream):
            flag = True
            break
    if (not flag):
        unitdef = cremodel_.createUnitDefinition()
        unitdef.setId(unit_stream)
        # Create individual unit objects that will be put inside the
        # UnitDefinition .
        if order != 0:
            unit = unitdef.createUnit()
            unit.setKind(UNIT_KIND_LITRE)
            unit.setExponent(order)
            unit.setMultiplier(1)
            unit.setScale(0)
            unit = unitdef.createUnit()
            unit.setKind(UNIT_KIND_MOLE)
            unit.setExponent(-order)
            unit.setMultiplier(1)
            unit.setScale(-3)

        unit = unitdef.createUnit()
        unit.setKind(UNIT_KIND_SECOND)
        unit.setExponent(-1)
        unit.setMultiplier(1)
        unit.setScale(0)
    return unit_stream

def Counter(items):
    return dict((i, items.count(i)) for i in items)

def getSubprd(cremodel_, mobjEnz, type, neighborslist):
    
    if type == "sub":
        reacSub = neighborslist
        reacSubCou = Counter(reacSub)

        # print " reacSubCou ",reacSubCou,"()",len(reacSubCou)
        noofSub = len(reacSubCou)
        rate_law = " "
        if reacSub:
            rate_law = processRateLaw(
                reacSubCou, cremodel_, noofSub, "sub", mobjEnz)
            return len(reacSub), rate_law
        else:
            return 0, rate_law
    elif type == "prd":
        reacPrd = neighborslist
        reacPrdCou = Counter(reacPrd)
        noofPrd = len(reacPrdCou)
        rate_law = " "
        if reacPrd:
            rate_law = processRateLaw(
                reacPrdCou, cremodel_, noofPrd, "prd", mobjEnz)
            return len(reacPrd), rate_law
    elif type == "enz"  or type == "chan":
        enzModifier = neighborslist
        enzModCou = Counter(enzModifier)
        noofMod = len(enzModCou)
        rate_law = " "
        if enzModifier:
            rate_law = processRateLaw(
                enzModCou, cremodel_, noofMod, "Modifier", mobjEnz)
            return len(enzModifier), rate_law


def processRateLaw(objectCount, cremodel, noofObj, typesp, mobjEnz):
    rate_law = ""
    nameList_[:] = []
    for value, count in objectCount.items():
        value = moose.element(value)
        '''
        if re.search("_xfer_",value.name):
            modelRoot = value.path[0:value.path.index('/',1)]
            xrefPool = value.name[:value.name.index("_xfer_")]
            xrefCompt = value.name[value.name.index("_xfer_") + len("_xfer_"):]
            orgCompt = moose.wildcardFind(modelRoot+'/##[FIELD(name)='+xrefCompt+']')[0]
            orgPool = moose.wildcardFind(orgCompt.path+'/##[FIELD(name)='+xrefPool+']')[0]
            value = orgPool
        '''
        nameIndex = value.name + "_" + \
            str(value.idValue) + "_" + str(value.dataIndex) + "_"
        clean_name = (str(idBeginWith(convertSpecialChar(nameIndex))))
        if mobjEnz == True:
            nameList_.append(clean_name)

        if typesp == "sub":
            sbmlRef = cremodel.createReactant()
        elif typesp == "prd":
            sbmlRef = cremodel.createProduct()
        elif typesp == "Modifier":
            sbmlRef = cremodel.createModifier()
            sbmlRef.setSpecies(clean_name)

        if typesp == "sub" or typesp == "prd":
            sbmlRef.setSpecies(clean_name)
            sbmlRef.setStoichiometry(count)
            if clean_name in spe_constTrue:
                sbmlRef.setConstant(True)
            else:
                sbmlRef.setConstant(False)
        if (count == 1):
            if rate_law == "":
                rate_law = clean_name
            else:
                rate_law = rate_law + "*" + clean_name
        else:
            if rate_law == "":
                rate_law = clean_name + "^" + str(count)
            else:
                rate_law = rate_law + "*" + clean_name + "^" + str(count)
    return(rate_law)


def listofname(reacSub, mobjEnz):
    objectCount = Counter(reacSub)
    nameList_[:] = []
    for value, count in objectCount.items():
        value = moose.element(value)
        nameIndex = value.name + "_" + \
            str(value.idValue) + "_" + str(value.dataIndex) + "_"
        clean_name = convertSpecialChar(nameIndex)
        if mobjEnz == True:
            nameList_.append(clean_name)


def writeReac(modelpath, cremodel_, sceneitems,reacGroup):
    for reac in moose.wildcardFind(modelpath + '/##[0][ISA=ReacBase]'):
        reacSub = reac.neighbors["sub"]
        reacPrd = reac.neighbors["prd"]
        if (len(reacSub) != 0 and len(reacPrd) != 0):

            reaction = cremodel_.createReaction()
            reacannoexist = False
            reacGpname = " "
            cleanReacname = convertSpecialChar(reac.name)
            setId = str(idBeginWith(cleanReacname +
                                           "_" +
                                           str(reac.idValue) +
                                           "_" +
                                           str(reac.dataIndex) +
                                           "_"))
            reaction.setId(setId)
            reaction.setName(str(idBeginWith(convertSpecialCharshot(reac.name))))
            #reaction.setName(cleanReacname)
            #Kf = reac.numKf
            #Kb = reac.numKb
            Kf = reac.Kf
            Kb = reac.Kb
            if Kb == 0.0:
                reaction.setReversible(False)
            else:
                reaction.setReversible(True)

            reaction.setFast(False)
            if moose.exists(reac.path + '/info'):
                Anno = moose.Annotator(reac.path + '/info')
                if Anno.notes != "":
                    cleanNotesR = convertNotesSpecialChar(Anno.notes)
                    notesStringR = "<body xmlns=\"http://www.w3.org/1999/xhtml\">\n \t \t" + \
                        cleanNotesR + "\n\t </body>"
                    reaction.setNotes(notesStringR)
                element = moose.element(reac)
                ele = getGroupinfo(element)
                if element.className == "Neutral" or sceneitems or Anno.x or Anno.y:
                    reacannoexist = True
                if reacannoexist:
                    reacAnno = "<moose:ModelAnnotation>\n"
                    if ele.className == "Neutral":
                        #reacAnno = reacAnno + "<moose:Group>" + ele.name + "</moose:Group>\n"
                        if ele not in reacGroup:
                            reacGroup[ele]=[setId]
                        else:
                            reacGroup[ele].append(setId)
                    if sceneitems:
                        #Saved from GUI, then scene co-ordinates are passed
                        reacAnno = reacAnno + "<moose:xCord>" + \
                                str(sceneitems[reac]['x']) + "</moose:xCord>\n" + \
                                "<moose:yCord>" + \
                                str(sceneitems[reac]['y'])+ "</moose:yCord>\n"
                    else:
                        #Saved from cmdline,genesis coordinates are kept as its
                        # SBML, cspace, python, then auto-coordinates are done
                        #and coordinates are updated in moose Annotation field
                        reacAnno = reacAnno + "<moose:xCord>" + \
                                str(Anno.x) + "</moose:xCord>\n" + \
                                "<moose:yCord>" + \
                                str(Anno.y)+ "</moose:yCord>\n"
                    if Anno.color:
                        reacAnno = reacAnno + "<moose:bgColor>" + Anno.color + "</moose:bgColor>\n"
                    if Anno.textColor:
                        reacAnno = reacAnno + "<moose:textColor>" + \
                            Anno.textColor + "</moose:textColor>\n"
                    reacAnno = reacAnno + "</moose:ModelAnnotation>"
                    reaction.setAnnotation(reacAnno)

            kl_s, sRL, pRL, compt = "", "", "", ""

            if not reacSub and not reacPrd:
                print(" Reaction ", reac.name, "missing substrate and product, this is not allowed in SBML which will not be written")
            else:
                kfl = reaction.createKineticLaw()
                if reacSub:
                    noofSub, sRateLaw = getSubprd(
                        cremodel_, False, "sub", reacSub)
                    if noofSub:
                        comptVec = findCompartment(moose.element(reacSub[0]))
                        if not moose.element(comptVec).isA("ChemCompt"):
                            print(comptVec.name+ " is not instance of ChemCompt") 
                            return -2
                        else:
                            compt = comptVec.name + "_" + \
                                str(comptVec.idValue) + "_" + \
                                str(comptVec.dataIndex) + "_"
                        cleanReacname = cleanReacname + "_" + \
                            str(reac.idValue) + "_" + \
                            str(reac.dataIndex) + "_"
                        kfparm = idBeginWith(cleanReacname) + "_" + "Kf"
                        sRL = compt + " * " + \
                            idBeginWith(cleanReacname) + "_Kf * " + sRateLaw
                        unit = parmUnit(noofSub - 1, cremodel_)
                        printParameters(kfl, kfparm, Kf, unit)
                        #kl_s = compt+"(" +sRL
                        kl_s = sRL
                    else:
                        print(reac.name + " has no substrate")
                        return -2
                else:
                    print(" Substrate missing for reaction ", reac.name)

                if reacPrd:
                    noofPrd, pRateLaw = getSubprd(
                        cremodel_, False, "prd", reacPrd)
                    if noofPrd:
                        if Kb:
                            kbparm = idBeginWith(cleanReacname) + "_" + "Kb"
                            pRL = compt + " * " + \
                                idBeginWith(cleanReacname) + \
                                "_Kb * " + pRateLaw
                            unit = parmUnit(noofPrd - 1, cremodel_)
                            printParameters(kfl, kbparm, Kb, unit)
                            #kl_s = kl_s+ "- "+pRL+")"
                            kl_s = kl_s + "-" + pRL
                    else:
                        print(reac.name + " has no product")
                        return -2
                else:
                    print(" Product missing for reaction ", reac.name)
            kfl.setFormula(kl_s)
            kfl.setNotes(
                "<body xmlns=\"http://www.w3.org/1999/xhtml\">\n\t\t" +
                kl_s +
                "\n \t </body>")
            kfl.setFormula(kl_s)

        else:
            print(" Reaction ", reac.name, "missing substrate and product, this is not allowed in SBML which will not be written")


def writeFunc(modelpath, cremodel_):
    funcs = moose.wildcardFind(modelpath + '/##[0][ISA=Function]')
    # if func:
    foundFunc = False
    for func in funcs:
        if func:
            #not neccessary parent is compartment can be group also
            if func.parent.className == "CubeMesh" or func.parent.className == "CyclMesh":
                if len(moose.element(func).neighbors["valueOut"]) > 0:
                    funcEle = moose.element(
                        moose.element(func).neighbors["valueOut"][0])
                    funcEle1 = moose.element(funcEle)
                    fName = idBeginWith(convertSpecialChar(
                    funcEle.name + "_" + str(funcEle.idValue) + "_" + str(funcEle.dataIndex) + "_"))
                    expr = " "
                    expr = str(moose.element(func).expr)
                    if expr:
                        foundFunc = True
                        item = func.path + '/x[0]'
                        sumtot = moose.element(item).neighbors["input"]
                        for i in range(0, len(sumtot)):
                            v = "x" + str(i)
                            if v in expr:
                                z = str(idBeginWith(str(convertSpecialChar(sumtot[i].name + "_" + str(moose.element(
                                    sumtot[i]).idValue) + "_" + str(moose.element(sumtot[i]).dataIndex)) + "_")))
                                expr = expr.replace(v, z)
            else:
                foundFunc = True
                fName = idBeginWith(convertSpecialChar(func.parent.name +
                                                       "_" + str(func.parent.idValue) +
                                                       "_" + str(func.parent.dataIndex) +
                                                       "_"))
                item = func.path + '/x[0]'
                sumtot = moose.element(item).neighbors["input"]
                expr = moose.element(func).expr
                for i in range(0, len(sumtot)):
                    v = "x" + str(i)
                    if v in expr:
                        z = str(idBeginWith(str(convertSpecialChar(sumtot[i].name + "_" + str(moose.element(
                            sumtot[i]).idValue) + "_" + str(moose.element(sumtot[i]).dataIndex)) + "_")))
                        expr = expr.replace(v, z)
            if foundFunc:
                rule = cremodel_.createAssignmentRule()
                rule.setVariable(fName)
                rule.setFormula(expr)

def convertNotesSpecialChar(str1):
    d = {"&": "_and", "<": "_lessthan_", ">": "_greaterthan_", "BEL": "&#176"}
    for i, j in d.items():
        str1 = str1.replace(i, j)
    # stripping \t \n \r and space from begining and end of string
    str1 = str1.strip(' \t\n\r')
    return str1

def getGroupinfo(element):
    #   Note: At this time I am assuming that if group exist (incase of Genesis)
    #   1. for 'pool' its between compartment and pool, /modelpath/Compartment/Group/pool
    #   2. for 'enzComplx' in case of ExpilcityEnz its would be, /modelpath/Compartment/Group/Pool/Enz/Pool_cplx
    #   For these cases I have checked, but subgroup may exist then this bit of code need to cleanup further down
    #   if /modelpath/Compartment/Group/Group1/Pool, then I check and get Group1
    #   And /modelpath is also a NeutralObject,I stop till I find Compartment

    while not mooseIsInstance(element, ["Neutral", "CubeMesh", "CylMesh","EndoMesh","NeuroMesh"]):
        element = element.parent
    return element


def idBeginWith(name):
    changedName = name
    if name[0].isdigit():
        changedName = "_" + name
    return changedName

def findGroup_compt(melement):
    while not (mooseIsInstance(melement, ["Neutral","CubeMesh", "CylMesh","EndoMesh","NeuroMesh"])):
        melement = melement.parent
    return melement

def convertSpecialCharshot(str1):
    d = { "BEL"     : "&#176", 
            "'"     : "_prime_",
            "\\"    : "_slash_",
            "/"     : "_slash_", 
            "["     : "_sbo_", 
            "]"     : "_sbc_",
            ": "    : "_" , 
            " "     : "_" }
    for i, j in d.items():
        str1 = str1.replace(i, j)
    return str1

def convertSpecialChar(str1):
    d = {"&": "_and", "<": "_lessthan_", ">": "_greaterthan_", "BEL": "&#176", "-": "_minus_", "'": "_prime_",
         "+": "_plus_", "*": "_star_", "/": "_slash_", "(": "_bo_", ")": "_bc_",
         "[": "_sbo_", "]": "_sbc_", ".": "_dot_", " ": "_"
         }
    for i, j in d.items():
        str1 = str1.replace(i, j)
    return str1


def writeSpecies(modelpath, cremodel_, sbmlDoc, sceneitems,speGroup):
    # getting all the species
    for spe in moose.wildcardFind(modelpath + '/##[0][ISA=PoolBase]'):
        #Eliminating xfer molecules writting
        if not re.search("_xfer_",spe.name):
            
            sName = convertSpecialChar(spe.name)
            comptVec = findCompartment(spe)
            speciannoexist = False
            
            if not comptVec.isA("ChemCompt"):
                return -2
            else:
                compt = comptVec.name + "_" + \
                    str(comptVec.idValue) + "_" + \
                    str(comptVec.dataIndex) + "_"
                s1 = cremodel_.createSpecies()
                spename = sName + "_" + \
                    str(spe.idValue) + "_" + str(spe.dataIndex) + "_"
                spename = str(idBeginWith(spename))
                s1.setId(spename)

                if spename.find(
                        "cplx") != -1 and spe.parent.isA("EnzBase"):
                    enz = spe.parent
                    if not moose.exists(spe.path+'/info'):
                        cplxinfo = moose.Annotator(spe.path+'/info')
                        enzpath = moose.element(spe.parent.path+'/info')
                        cplxinfo.x = moose.element(enzpath).x
                        cplxinfo.y = int((moose.element(enzpath).y))+10
                 
                    if (moose.element(enz.parent), moose.PoolBase):
                        # print " found a cplx name ",spe.parent,
                        # moose.element(spe.parent).parent
                        enzname = enz.name
                        enzPool = (enz.parent).name
                        sName = convertSpecialChar(
                            enzPool + "_" + enzname + "_" + sName)

                #s1.setName(sName)
                s1.setName(str(idBeginWith(convertSpecialCharshot(spe.name))))
                # s1.setInitialAmount(spe.nInit)
                s1.setInitialConcentration(spe.concInit)
                s1.setCompartment(compt)
                #  Setting BoundaryCondition and constant as per this rule for BufPool
                #  -constanst  -boundaryCondition  -has assignment/rate Rule  -can be part of sub/prd
                #   false           true              yes                       yes
                #   true            true               no                       yes
                if spe.className == "BufPool" or spe.className == "ZombieBufPool":
                    # BoundaryCondition is made for buff pool
                    s1.setBoundaryCondition(True)

                    if moose.exists(spe.path + '/func'):
                        bpf = moose.element(spe.path)
                        for fp in bpf.children:
                            if fp.className == "Function" or fp.className == "ZombieFunction":
                                if len(moose.element(
                                        fp.path + '/x').neighbors["input"]) > 0:
                                    s1.setConstant(False)
                                else:
                                    # if function exist but sumtotal object doesn't
                                    # exist
                                    spe_constTrue.append(spename)
                                    s1.setConstant(True)
                    else:
                        spe_constTrue.append(spename)
                        s1.setConstant(True)
                else:
                    # if not bufpool then Pool, then
                    s1.setBoundaryCondition(False)
                    s1.setConstant(False)
                s1.setUnits("substance")
                s1.setHasOnlySubstanceUnits(False)

                if moose.exists(spe.path + '/info'):
                    Anno = moose.Annotator(spe.path + '/info')
                    if Anno.notes != "":
                        cleanNotesS = convertNotesSpecialChar(Anno.notes)
                        notesStringS = "<body xmlns=\"http://www.w3.org/1999/xhtml\">\n \t \t" + \
                            cleanNotesS + "\n\t </body>"
                        s1.setNotes(notesStringS)


                    element = moose.element(spe)
                    ele = getGroupinfo(element)
                    
                    speciAnno = "<moose:ModelAnnotation>\n"
                    if ele.className == "Neutral":
                        #speciAnno = speciAnno + "<moose:Group>" + ele.name + "</moose:Group>\n"
                        if ele not in speGroup:
                            speGroup[ele]=[spename]
                        else:
                            speGroup[ele].append(spename)


                    if sceneitems:
                        #Saved from GUI, then scene co-ordinates are passed
                        speciAnno = speciAnno + "<moose:xCord>" + \
                                str(sceneitems[spe]['x']) + "</moose:xCord>\n" + \
                                "<moose:yCord>" + \
                                str(sceneitems[spe]['y'])+ "</moose:yCord>\n"
                    else:
                        #Saved from cmdline,genesis coordinates are kept as its
                        # SBML, cspace, python, then auto-coordinates are done
                        #and coordinates are updated in moose Annotation field
                        speciAnno = speciAnno + "<moose:xCord>" + \
                                str(Anno.x) + "</moose:xCord>\n" + \
                                "<moose:yCord>" + \
                                str(Anno.y)+ "</moose:yCord>\n"
                    if Anno.color:
                        speciAnno = speciAnno + "<moose:bgColor>" + Anno.color + "</moose:bgColor>\n"
                    if Anno.textColor:
                        speciAnno = speciAnno + "<moose:textColor>" + \
                            Anno.textColor + "</moose:textColor>\n"
                    
                    speciAnno = speciAnno + "<moose:diffConstant>" + str(spe.diffConst) + "</moose:diffConstant>\n"
                    speciAnno = speciAnno + "<moose:motorConstant>" + str(spe.motorConst)+ "</moose:motorConstant>\n" 
                    speciAnno = speciAnno + "</moose:ModelAnnotation>"
                    s1.setAnnotation(speciAnno)
                else:
                    #E.g cplx doesn't have info field but findsim layout expecting diffConstant and motorConstant.
                    speciAnno = "<moose:ModelAnnotation>\n"
                    speciAnno = speciAnno + "<moose:diffConstant>" + str(0.0) + "</moose:diffConstant>\n"
                    speciAnno = speciAnno + "<moose:motorConstant>" + str(0.0)+ "</moose:motorConstant>\n" 
                    speciAnno = speciAnno + "</moose:ModelAnnotation>"
                    s1.setAnnotation(speciAnno)
    return True


def writeCompt(modelpath, cremodel_):
    # getting all the compartments
    compts = moose.wildcardFind(modelpath + '/##[0][ISA=ChemCompt]')
    groupInfo = {}
    comptID_sbml = {}
    for compt in compts:
        comptAnno = ""
        comptName = convertSpecialChar(compt.name)
        # converting m3 to litre
        createCompt = True
        size = compt.volume * pow(10, 3)
        ndim = compt.numDimensions
        csetId = (str(idBeginWith(comptName +
                                "_" +
                                str(compt.idValue) +
                                "_" +
                                str(compt.dataIndex) +
                                "_")))
        comptID_sbml[compt] = csetId
        if compt.isA("EndoMesh"):
            if comptID_sbml.get(compt.surround) == None:
                createCompt = False
                return False,groupInfo,"Outer compartment need to be specified for EndoMesh "
                #return " EndoMesh need to outer compartment to be specified "
            else:
                comptAnno = "<moose:CompartmentAnnotation><moose:Mesh>" + \
                                    str(compt.className) + "</moose:Mesh>\n" + \
                                    "<moose:surround>" + \
                                    str(comptID_sbml[compt.surround])+ "</moose:surround>\n" + \
                                    "<moose:isMembraneBound>" + \
                                    str(compt.isMembraneBound)+ "</moose:isMembraneBound>\n" 
                if moose.exists(compt.path+'/info'):
                    if moose.element(compt.path+'/info').notes != "":
                        comptAnno = comptAnno + "<moose:Notes>" \
                        + moose.element(compt.path+'/info').notes + "</moose:Notes>"
                comptAnno = comptAnno+ "</moose:CompartmentAnnotation>"
        elif compt.isA("CylMesh") :
            size = (compt.volume/compt.numDiffCompts)*pow(10,3)
            comptAnno = "<moose:CompartmentAnnotation><moose:Mesh>" + \
                                str(compt.className) + "</moose:Mesh>\n" + \
                                "<moose:totLength>" + \
                                str(compt.totLength)+ "</moose:totLength>\n" + \
                                "<moose:diffLength>" + \
                                str(compt.diffLength)+ "</moose:diffLength>\n" + \
                                "<moose:isMembraneBound>" + \
                                str(compt.isMembraneBound)+ "</moose:isMembraneBound>\n"
            if moose.exists(compt.path+'/info'):
                if moose.element(compt.path+'/info').notes != "":
                    comptAnno = comptAnno + "<moose:Notes>" \
                    + moose.element(compt.path+'/info').notes + "</moose:Notes>"
            comptAnno = comptAnno+ "</moose:CompartmentAnnotation>"
        else:
            comptAnno = "<moose:CompartmentAnnotation><moose:Mesh>" + \
                                str(compt.className) + "</moose:Mesh>\n" + \
                                "<moose:isMembraneBound>" + \
                                str(compt.isMembraneBound)+ "</moose:isMembraneBound>\n" 
            if moose.exists(compt.path+'/info'):
                if moose.element(compt.path+'/info').notes != "":
                    comptAnno = comptAnno + "<moose:Notes>" \
                    + moose.element(compt.path+'/info').notes + "</moose:Notes>"
            comptAnno = comptAnno+ "</moose:CompartmentAnnotation>"
        if createCompt:
            c1 = cremodel_.createCompartment()
            c1.setId(csetId)
            c1.setName(comptName)
            c1.setConstant(True)
            c1.setSize(compt.volume*pow(10,3))
            #c1.setSize(size)
            if compt.isA("EndoMesh"):
                c1.outside = comptID_sbml[compt.surround]
                
            if comptAnno:
                c1.setAnnotation(comptAnno)
            c1.setSpatialDimensions(ndim)
            if ndim == 3:
                c1.setUnits('volume')
            elif ndim == 2:
                c1.setUnits('area')
            elif ndim == 1:
                c1.setUnits('metre')
            
            #For each compartment get groups information along
            for grp in moose.wildcardFind(compt.path+'/##[0][TYPE=Neutral]'):
                grp_cmpt = findGroup_compt(grp.parent)
                try:
                    value = groupInfo[moose.element(grp)]
                except KeyError:
                    # Grp is not present
                    groupInfo[moose.element(grp)] = []
        

    if compts:
        return True,groupInfo,""
    else:
        return False,groupInfo,""
# write Simulation runtime,simdt,plotdt

def writeSimulationAnnotation(modelpath):
    modelAnno = ""
    plots = ""
    if moose.exists(modelpath + '/info'):
        mooseclock = moose.Clock('/clock')
        modelAnno = "<moose:ModelAnnotation>\n"
        modelAnnotation = moose.element(modelpath + '/info')
        modelAnno = modelAnno + "<moose:runTime> " + \
            str(modelAnnotation.runtime) + " </moose:runTime>\n"
        modelAnno = modelAnno + "<moose:solver> " + \
            modelAnnotation.solver.replace(" ","") + " </moose:solver>\n"
        modelAnno = modelAnno + "<moose:simdt>" + \
            str(mooseclock.dts[12]) + " </moose:simdt>\n"
        modelAnno = modelAnno + "<moose:plotdt> " + \
            str(mooseclock.dts[18]) + " </moose:plotdt>\n"
        plots = ""
        graphs = moose.wildcardFind(modelpath + "/##[0][TYPE=Table2]")
        
        for gphs in range(0, len(graphs)):
            gpath = graphs[gphs].neighbors['requestOut']
            if len(gpath) != 0:
                q = moose.element(gpath[0])
                ori = q.path
                name = convertSpecialChar(q.name)
                graphSpefound = False
                
                #while not(isinstance(moose.element(q), moose.CubeMesh) or isinstance(moose.element(q),moose.CylMesh) \
                #    or    isinstance(moose.element(q), moose.EndoMesh) or isinstance(moose.element(q),moose.NeuroMesh)):
                #    q = q.parent
                #    graphSpefound = True
                if graphSpefound:
                    if not plots:
                        plots = ori[ori.find(q.name)-1:len(ori)]
                        #plots = '/' + q.name + '/' + name

                    else:
                        plots = plots + "; "+ori[ori.find(q.name)-1:len(ori)]
                        #plots = plots + "; /" + q.name + '/' + name
                
        if plots != " ":
            modelAnno = modelAnno + "<moose:plots> " + plots + "</moose:plots>\n"
        modelAnno = modelAnno + "</moose:ModelAnnotation>"
        
    return modelAnno

def recalculatecoordinates(modelpath, mObjlist,xcord,ycord):
    positionInfoExist = not(len(np.nonzero(xcord)[0]) == 0 \
                        and len(np.nonzero(ycord)[0]) == 0)
    defaultsceneWidth =  1000
    defaultsceneHeight =  800
    if positionInfoExist:
        #Here all the object has been taken now recalculate and reassign back x and y co-ordinates
        xmin = min(xcord)
        xmax = max(xcord)
        ymin = min(ycord)
        ymax = max(ycord)
        for merts in mObjlist:
            objInfo = merts.path+'/info'
            if moose.exists(objInfo):
                Ix = defaultsceneWidth * ((xyPosition(objInfo,'x')-xmin)/(xmax-xmin))
                Iy = defaultsceneHeight * ((xyPosition(objInfo,'y')-ymin)/(ymax-ymin))
                moose.element(objInfo).x = Ix
                moose.element(objInfo).y = Iy
   
    else:
        srcdesConnection = {}
        setupItem(modelpath,srcdesConnection)
        meshEntry,xmin,xmax,ymin,ymax,positionInfoExist,sceneitems = setupMeshObj(modelpath)
        if not positionInfoExist:

            sceneitems = autoCoordinates(meshEntry,srcdesConnection)
        sceneitems = autoCoordinates(meshEntry,srcdesConnection)

        #print srcdesConnection
        '''
        #meshEntry,xmin,xmax,ymin,ymax,positionInfoExist,sceneitems = setupMeshObj(modelpath)
        #if not positionInfoExist:
            #cmin,cmax,sceneitems = autoCoordinates(meshEntry,srcdesConnection)
            sceneitems = autoCoordinates(meshEntry,srcdesConnection)
	'''        
def writeUnits(cremodel_):
    unitVol = cremodel_.createUnitDefinition()
    unitVol.setId("volume")
    unit = unitVol.createUnit()
    unit.setKind(UNIT_KIND_LITRE)
    unit.setMultiplier(1.0)
    unit.setExponent(1.0)
    unit.setScale(0)

    unitSub = cremodel_.createUnitDefinition()
    unitSub.setId("substance")
    unit = unitSub.createUnit()
    unit.setKind(UNIT_KIND_MOLE)
    unit.setMultiplier(1)
    unit.setExponent(1.0)
    unit.setScale(-3)

    unitLen = cremodel_.createUnitDefinition()
    unitLen.setId("length")
    unit = unitLen.createUnit()
    unit.setKind(UNIT_KIND_METRE)
    unit.setMultiplier(1.0)
    unit.setExponent(1.0)
    unit.setScale(0)

    unitArea = cremodel_.createUnitDefinition()
    unitArea.setId("area")
    unit = unitArea.createUnit()
    unit.setKind(UNIT_KIND_METRE)
    unit.setMultiplier(1.0)
    unit.setExponent(2.0)
    unit.setScale(0)

    unitTime = cremodel_.createUnitDefinition()
    unitTime.setId("time")
    unit = unitTime.createUnit()
    unit.setKind(UNIT_KIND_SECOND)
    unit.setExponent(1)
    unit.setMultiplier(1)
    unit.setScale(0)

if __name__ == "__main__":
    try:
        sys.argv[1]
    except IndexError:
        print("Filename or path not given")
        exit(0)
    else:
        filepath = sys.argv[1]
        if not os.path.exists(filepath):
            print("Filename or path does not exist",filepath)
        else:
            try:
                sys.argv[2]
            except :
                modelpath = filepath[filepath.rfind('/'):filepath.find('.')]
            else:
                modelpath = sys.argv[2]

            moose.loadModel(filepath, modelpath, "gsl")

            written, c, writtentofile = mooseWriteSBML(modelpath, filepath)
            if written:
                print(" File written to ", writtentofile)
            else:
                print(" could not write model to SBML file")



# -*- coding: utf-8 -*-
'''
Here is some docstring.
'''
from __future__ import print_function, division

import os
import random
import time
import multiprocessing as mp
import socket
import moose
import json
import struct
import numpy as np
import moose.utils as mu


print('Using from %s. VERSION: %s' % (moose.__file__, moose.__version__))

sockPath = '/tmp/MOOSE'
os.environ['MOOSE_STREAMER_ADDRESS'] = 'file://%s' % sockPath


def streamer_handler(done, q):
    # Streamer handler
    global sockPath
    global all_done_
    s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    while not os.path.exists(sockPath):
        time.sleep(0.01)
        if done.value == 1:
            break
    s.connect( sockPath )
    print( "[INFO ] Connected" )
    s.settimeout(1e-5)
    data = b''
    while True:
        if done.value == 1:
            break
        try:
            d = s.recv(10240)
            if d.strip():
                data += d
        except socket.timeout:
            pass

    res = mu.decode_data(data)
    q.put(res)

def make_network():
    """
    This snippet sets up a recurrent network of IntFire objects, using
    SimpleSynHandlers to deal with spiking events.
    It isn't very satisfactory as activity runs down after a while.
    It is a good example for using the IntFire, setting up random
    connectivity, and using SynHandlers.
    """
    global all_done_
    done = mp.Value('i', 0)
    q = mp.Queue()
    th = mp.Process(target=streamer_handler, args=(done, q))
    th.start()

    size = 1024
    dt = 0.2
    runsteps = 10
    delayMin = 0
    delayMax = 4
    weightMax = 1
    Vmax = 1.0
    thresh = 0.4
    refractoryPeriod = 0.4
    tau = 0.5
    connectionProbability = 0.01
    random.seed( 123 )
    np.random.seed( 456 )
    t0 = time.time()

    network = moose.IntFire( 'network', size );
    syns = moose.SimpleSynHandler( '/network/syns', size );
    moose.connect( syns, 'activationOut', network, 'activation', 'OneToOne' )
    moose.le( '/network' )
    syns.vec.numSynapses = [1] * size
    sv = moose.vec( '/network/syns/synapse' )
    print('before connect t = %.3f'%(time.time() - t0))
    mid = moose.connect( network, 'spikeOut', sv, 'addSpike', 'Sparse')
    print('after connect t = %.3f'%(time.time() - t0))
    #print mid.destFields
    m2 = moose.element( mid )
    m2.setRandomConnectivity( connectionProbability, 5489 )
    print('after setting connectivity, t=%.3f'%(time.time() - t0))
    #network.vec.Vm = [(Vmax*random.random()) for r in range(size)]
    network.vec.Vm = np.random.rand( size ) * Vmax
    network.vec.thresh = thresh
    network.vec.refractoryPeriod = refractoryPeriod
    network.vec.tau = tau
    numSynVec = syns.vec.numSynapses
    print('Middle of setup, t = %.3f'%(time.time() - t0))
    numTotSyn = sum( numSynVec )
    print((numSynVec.size, ', tot = ', numTotSyn,  ', numSynVec = ', numSynVec))
    for item in syns.vec:
        h = moose.element(item)
        h.synapse.delay = delayMin + (delayMax-delayMin) * np.random.rand(len(h.synapse))
        h.synapse.weight = np.random.rand(len(h.synapse)) * weightMax
    print('After setup, t = %.3f'%(time.time()-t0))

    numStats = 100
    stats = moose.SpikeStats( '/stats', numStats )
    stats.vec.windowLength = 1 # timesteps to put together.
    plots = moose.Table( '/plot', numStats )
    convergence = size // numStats
    for i in range( numStats ):
        for j in range( size//numStats ):
            k = i * convergence + j
            moose.connect( network.vec[k], 'spikeOut', stats.vec[i], 'addSpike' )
    moose.connect( plots, 'requestOut', stats, 'getMean', 'OneToOne' )

    t1 = time.time()
    moose.reinit()
    print('reinit time t = %.3f'%(time.time() - t1))
    network.vec.Vm = np.random.rand( size ) * Vmax
    print('setting Vm , t = %.3f'%(time.time() - t1))
    t1 = time.time()
    moose.start(runsteps * dt, 1)
    time.sleep(0.1)
    done.value = 1
    print('runtime, t = %.3f'%(time.time() - t1))
    print(network.vec.Vm[99:103], network.vec.Vm[900:903])
    res = q.get()

    for tabPath in res:
        aWithTime = res[tabPath]
        a = aWithTime[1::2]
        b = moose.element(tabPath).vector
        print( tabPath, len(a), len(b) )
        if len(a) == len(b):
            assert np.equal(a, b).all()
        else:
            print( "Table did not equal size. The last table is allowed to "
                    " have fewer entries." )


    th.join()
    print( 'All done' )

if __name__ == '__main__':
    make_network()



# -*- coding: utf-8 -*-

# This test shows how to properly use metaclass to avoid 
# > TypeError: metaclass conflict: the metaclass of a derived class must be 
# > a (non-strict) subclass of the metaclasses of all its bases
# error.

import moose
try:
    import six
except ImportError:
    print("[INFO ] Requires `six` to run this test.")
    quit(0)

# Can't use type here. Use metaclass of the base class (i.e.
# moose.melement).
class ChannelMeta(moose.melement.__class__):
    def __new__(cls, name, bases, cdict):     
        if  'abstract' in cdict and cdict['abstract'] == True:
            return type.__new__(cls, name, bases, cdict)


@six.add_metaclass(ChannelMeta)
class ChannelBase(moose.HHChannel):
    annotation = {'cno': 'cno_0000047'}
    abstract = True
    def __init__(self, path, xpower=1, ypower=0, Ek=0.0):
        moose.HHChannel.__init__(self, path)

def test_metaclass():
    a = ChannelBase('a')
    print(a)

def main():
    test_metaclass()

if __name__ == '__main__':
    main()
